/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: compose_attachment.c
 *   	 Mail composition attachment functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "eucalyptus.h"

CompAttachment *compose_attachment_new(void)
{
	CompAttachment *catt = NULL;
	
	if(catt = g_malloc0(sizeof(CompAttachment)))
	{
		strncpy(catt->ContentType, "application/octet-stream", sizeof(catt->ContentType) - 1);
		
		catt->Type = CONTENTTYPE_APPLICATION | CONTENTSUBTYPE_APPLICATION_OCTETSTREAM;
		catt->Encoding = CAE_BASE64QP;
	}
	
	return catt;
}

void compose_attachment_free(CompAttachment *catt)
{
	if(!catt)
		return;
	
	g_free(catt);
}

gboolean compose_create_attachment_window(Composition *comp, CompAttachment *catt)
{
	GtkWidget *table;
	gchar *attachment_mime_types[] =
	{
		"text/plain",
		"text/enriched",
		"text/html",
		"image/jpeg",
		"image/gif",
		"image/png",
		"audio/wav",
		"video/mpeg",
		"application/octet-stream",
		NULL,
	};

	g_return_val_if_fail(comp != NULL, FALSE);

	if(comp->att_info.Window)
		return;

	comp->att_info.catt = catt;

	/* Create the window.
	 */
	comp->att_info.Title = catt ? g_strdup_printf(_("Attachment: %s"), catt->Name) : g_strdup(_("New attachment"));
	comp->att_info.Window = gnome_dialog_new(comp->att_info.Title, 
			_("Attach"), 
			GNOME_STOCK_BUTTON_CANCEL, 
			NULL);
	
	/* Add a table.
	 */
	table = gtk_table_new(4, 2, FALSE);
	{
		glong i;
		guint top = 0;
		
		/* Attachment name.
		 */
		comp->att_info.Name = gtk_entry_new_with_max_length(200);
		if(catt)
			gtk_entry_set_text(GTK_ENTRY(comp->att_info.Name), catt->Name);
		gtk_signal_connect(GTK_OBJECT(comp->att_info.Name), "changed",
					 GTK_SIGNAL_FUNC(compose_attachinfo_changed_cb), comp);
		prefs_layout_table_item(table, comp->att_info.Name, _("Name"), &top, 0);

		/* Attachment file.
		 */
		comp->att_info.Filename = gnome_file_entry_new("attachment-filename", _("Select attachment"));
		if(catt)
			gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(comp->att_info.Filename))), catt->Filename);
		gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(comp->att_info.Name))), "changed",
					 GTK_SIGNAL_FUNC(compose_attachinfo_changed_cb), comp);
		prefs_layout_table_item(table, comp->att_info.Filename, _("Filename"), &top, 0);

		/* Attachment description.
		 */
		comp->att_info.Desc = gtk_entry_new_with_max_length(200);
		if(catt)
			gtk_entry_set_text(GTK_ENTRY(comp->att_info.Desc), catt->Description);
		gtk_signal_connect(GTK_OBJECT(comp->att_info.Desc), "changed",
					 GTK_SIGNAL_FUNC(compose_attachinfo_changed_cb), comp);
		prefs_layout_table_item(table, comp->att_info.Desc, _("Description"), &top, 0);

		/* Attachment content-type.
		 */
		comp->att_info.Type = gnome_entry_new("attachment-type");
		for(i=0; attachment_mime_types[i]; i++)
			gnome_entry_append_history(GNOME_ENTRY(comp->att_info.Type), FALSE, attachment_mime_types[i]);
		if(catt)
			gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(comp->att_info.Type))), catt->ContentType);
		gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(comp->att_info.Type))), "changed",
					 GTK_SIGNAL_FUNC(compose_attachinfo_changed_cb), comp);
		prefs_layout_table_item(table, comp->att_info.Type, _("Content-type"), &top, 0);
	}
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(comp->att_info.Window)->vbox), table, TRUE, TRUE, 0);

	gnome_dialog_set_close(GNOME_DIALOG(comp->att_info.Window), TRUE);

	/* Buttons.
	 */
	gnome_dialog_button_connect(GNOME_DIALOG(comp->att_info.Window), 0,
			GTK_SIGNAL_FUNC(compose_attachinfo_attach_cb), comp);
	gnome_dialog_button_connect_object(GNOME_DIALOG(comp->att_info.Window), 0,
			GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(comp->att_info.Window));
	gnome_dialog_button_connect_object(GNOME_DIALOG(comp->att_info.Window), 1,
			GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(comp->att_info.Window));

	/* Signals.
	 */
	gtk_signal_connect(GTK_OBJECT(comp->att_info.Window), "close",
			GTK_SIGNAL_FUNC(compose_attachinfo_close_cb), comp);

	/* Show it.
	 */
	gtk_widget_show_all(comp->att_info.Window);

	/* Update the "Attach" button.
	 */
	{ 
		   glong tlen = 0;
		   gchar *name, *fname, *type;
		   gboolean has_slash;

		   name = gtk_editable_get_chars(GTK_EDITABLE(comp->att_info.Name), 0, -1);
		   fname = gtk_editable_get_chars(GTK_EDITABLE(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(comp->att_info.Filename))), 0, -1);
		   type = gtk_editable_get_chars(GTK_EDITABLE(gnome_entry_gtk_entry(GNOME_ENTRY(comp->att_info.Type))), 0, -1);
		   tlen = strlen(type);
		   has_slash = !!strchr(type, '/');

		   gnome_dialog_set_sensitive(GNOME_DIALOG(comp->att_info.Window), 0, *name && *fname && (tlen > 2) && has_slash);

		   g_free(name);
		   g_free(fname);
		   g_free(type);
	}
}

/* Callbacks.
 */
void compose_attachinfo_changed_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;
	glong tlen = 0;
	gchar *name, *fname, *type;
	gboolean has_slash;

	name = gtk_editable_get_chars(GTK_EDITABLE(comp->att_info.Name), 0, -1);
	fname = gtk_editable_get_chars(GTK_EDITABLE(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(comp->att_info.Filename))), 0, -1);
	type = gtk_editable_get_chars(GTK_EDITABLE(gnome_entry_gtk_entry(GNOME_ENTRY(comp->att_info.Type))), 0, -1);
	tlen = strlen(type);
	has_slash = !!strchr(type, '/');

	if(*fname && !*type)
	{
		gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(comp->att_info.Type))),
				gnome_mime_type_of_file(fname));
	}
	else
		gnome_dialog_set_sensitive(GNOME_DIALOG(comp->att_info.Window), 
				0, *name && *fname && (tlen > 2) && has_slash);

	g_free(name);
	g_free(fname);
	g_free(type);
}

void compose_attachinfo_attach_cb(GnomeDialog *dialog, gpointer data)
{
	Composition *comp = (Composition *)data;
	CompAttachment *catt = NULL;
	gchar *text;
	gboolean adding = FALSE;

	if(!(catt = comp->att_info.catt))
	{
		catt = comp->att_info.catt = compose_attachment_new();
		adding = TRUE;
	}

	if(!catt)
	{
		   euc_request(comp->Window,
					    _("Eucalyptus error"),
					    _("Unable to create attachment!"),
					    euc_ok_button, 0,
					    EUCRF_ASYNC);
		   return;
	}

	text = gtk_editable_get_chars(GTK_EDITABLE(comp->att_info.Name), 0, -1);
	strncpy(catt->Name, text, sizeof(catt->Name) - 1);
	g_free(text);

	text = gtk_editable_get_chars(GTK_EDITABLE(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(comp->att_info.Filename))), 0, -1);
	strncpy(catt->Filename, text, sizeof(catt->Filename) - 1);
	g_free(text);

	text = gtk_editable_get_chars(GTK_EDITABLE(comp->att_info.Desc), 0, -1);
	strncpy(catt->Description, text, sizeof(catt->Description) - 1);
	g_free(text);

	text = gtk_editable_get_chars(GTK_EDITABLE(gnome_entry_gtk_entry(GNOME_ENTRY(comp->att_info.Type))), 0, -1);
	strncpy(catt->ContentType, text, sizeof(catt->ContentType) - 1);
	g_free(text);

	if(adding)
		   list_append(&comp->Attachments, catt);

	compose_populate_attachment_list(comp);

}

void compose_attachinfo_close_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	comp->att_info.Window = NULL;
	comp->att_info.catt = NULL;
	
	g_free(comp->att_info.Title);
	comp->att_info.Title = NULL;
	
	gtk_clist_unselect_all(GTK_CLIST(comp->w.AttachList));
	
	comp->SelectedAttachment = 0;
	gtk_widget_set_sensitive(comp->w.EditAtch, FALSE);
	gtk_widget_set_sensitive(comp->w.DelAtch, FALSE);
}


