/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: folder_monitor.c
 *   	 Folder monitor thread
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#if USE_FOLDER_MONITOR

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "eucalyptus.h"

struct MonitorNode
{
	Folder *folder;
	time_t last_mod;
	time_t last_change;
};

struct MonitorMsg
{
	eMsg msg;
	struct MonitorNode *mnode;
};

enum
{
	MSG_MONITOR_ADD=1,
	MSG_MONITOR_REMOVE
};

void *folder_monitor_thread(void *data)
{
	struct List fnodes;
	GList *node;
	gboolean done = FALSE;
	
	/* Initialize our list of nodes to watch.
	 */
	list_init(&fnodes);

	/* Start the loop.
	 */
	while(!done)
	{
		GList *node;
		
		/* Check for modifications to the list.
		 */
		pthread_mutex_lock(&euc.monitor.Mutex);
		for(node = g_list_first(euc.monitor.Msgs.List); node; node = g_list_first(euc.monitor.Msgs.List))
		{
			struct MonitorNode *mnode;
			struct MonitorMsg *mmsg;
			
			mmsg = (struct MonitorMsg *)node->data;
			euc.monitor.Msgs.List = list_remove(&euc.monitor.Msgs, mmsg);

			if(mmsg->msg.type == MSG_TYPE_DIE)
			{
				done = TRUE;
				break;
			}
			
			mnode = mmsg->mnode;
			g_free(mmsg);

			list_append(&fnodes, mnode);
		}
		pthread_mutex_unlock(&euc.monitor.Mutex);
		
		if(done)
			break;
		
		/* Traverse the list of nodes.
		 */
		for(node = g_list_first(fnodes.List); node; node = g_list_next(node))
		{
			struct MonitorNode *mnode;
			struct stat *st;
			glong count = 0;
			gchar *fname;
			
			mnode = (struct MonitorNode *)node->data;
			
			fname = folder_get_full_path(mnode->folder);
			if(st = path_stat(fname))
			{
				/* Check for initial or update.
				 */
				if(mnode->last_mod)
					mnode->last_mod = st->st_mtime;
				else if(st->st_mtime > mnode->last_mod)
				{
					struct MatchState ms;
					glong error = 0;
					
					/* Find out what's new.  If nothing is
					 * new, then assume that something was
					 * changed or deleted, and notify the
					 * folder that it should re-build its
					 * index.
					 */
					error = match_first(&ms, fname, select_msg_file);
					while(!error)
					{
						struct dirent *de;
						struct stat *st;
						
						de = ms.entry;
						st = ms.st;
						
						/* Check the modification time on
						 * the entry.
						 */
						if(st->st_mtime > mnode->last_mod)
						{
							NewMsg *nmsg;
							
							/* Increment the counter.
							 */
							count++;
							
							/* Send a message to the folder.
							 */
							if(nmsg = g_malloc0(sizeof(NewMsg)))
							{
								nmsg->msg.type = MSG_TYPE_NEW;
								nmsg->folder = mnode->folder;
								strncpy(nmsg->filename, de->d_name, sizeof(nmsg->filename) - 1);
								
								pthread_mutex_lock(&euc.q.Mutex);
								list_append(&euc.q.Msgs, nmsg);
								pthread_mutex_unlock(&euc.q.Mutex);
							}
						}
					
						error = match_next(&ms);
					}
					match_end(&ms);
					
					/* Check if we actually found a new
					 * message.  If not, then send a general
					 * update message to the folder, since
					 * something was change or deleted, which
					 * we can't track.
					 */
					if(!count)
					{
					}

					/* Update our timestamp.
					 */
					mnode->last_mod = st->st_mtime;
				}

				
				g_free(st);
			}
		}

		/* Go to sleep.
		 */		
		sleep(15);
	}
	
	/* Clean up.
	 */

	/* Purge items in message queue.
	 */
	pthread_mutex_lock(&euc.monitor.Mutex);
	for(node = g_list_first(euc.monitor.Msgs.List); node; node = g_list_first(euc.monitor.Msgs.List))
	{
		struct MonitorMsg *mmsg;
		
		mmsg = (struct MonitorMsg *)node->data;
		euc.monitor.Msgs.List = list_remove(&euc.monitor.Msgs, node);
		
		g_free(mmsg->mnode);
		g_free(mmsg);
	}
	pthread_mutex_unlock(&euc.monitor.Mutex);

	/* Purge folder nodes.
	 */
	for(node = g_list_first(fnodes.List); node; node = g_list_first(fnodes.List))
	{
		struct MonitorNode *mnode;
		
		mnode = (struct MonitorNode *)node->data;
		fnodes.List = list_remove(&fnodes, node);
		g_free(mnode);
	}
}

gboolean folder_monitor_start(void)
{
	GList *node;
	
	/* Initialize the monitor's mutex.
	 */
	pthread_mutex_init(&euc.monitor.Mutex, NULL);
	list_init(&euc.monitor.Msgs);
	
	/* Initialize and start the monitor thread.
	 */
	if(pthread_attr_init(&euc.monitor.Attr))
		return FALSE;
	pthread_attr_setdetachstate(&euc.monitor.Attr, PTHREAD_CREATE_DETACHED);
	
	if(pthread_create(&euc.monitor.Thread, &euc.monitor.Attr, folder_monitor_thread, NULL))
		return FALSE;
	
	/* Go through the list of folders, and
	 * add each one to the monitor.
	 */
	for(node = g_list_first(euc.Folders.List); node; node = g_list_next(node))
	{
		Folder *folder;
		
		folder = (Folder *)node->data;
		folder_monitor_modify_node(folder, FALSE);
	}
	
	return TRUE;
}

void folder_monitor_stop(void)
{
}

gboolean folder_monitor_modify_node(Folder *folder, gboolean remove)
{
	struct MonitorMsg *mmsg;
	struct MonitorNode *mnode;
	gboolean success = FALSE;
	
	g_return_val_if_fail(folder != NULL, FALSE);

	/* Allocate a node and msg.
	 */
	if(mmsg = g_malloc(sizeof(struct MonitorMsg)))
	{
		if(mnode = g_malloc0(sizeof(struct MonitorNode)))
		{
			mnode->folder = folder;

			/* Set up the message.
			 */
			mmsg->msg.type = remove ? MSG_MONITOR_REMOVE : MSG_MONITOR_ADD;
			mmsg->mnode = mnode;
			
			/* Add it the monitor's queue.
			 */
			pthread_mutex_lock(&euc.monitor.Mutex);
			list_append(&euc.monitor.Msgs, mmsg);
			pthread_mutex_unlock(&euc.monitor.Mutex);
		}
		else
		{
			g_free(mmsg);
			success = FALSE;
		}
	}
	else
		success = FALSE;
	
	return success;
}

#endif
