/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: folders.c
 *   	 Folder handling functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>

#include "eucalyptus.h"

Folder *folder_new(struct FolderData *fd)
{
	Folder *folder = NULL;

	if((folder = g_malloc0(sizeof(Folder))))
	{
		if(fd)
		{
			gchar *dir;
			
			/* Copy the data over.
			 */
			memcpy(&folder->Data, fd, sizeof(struct FolderData));
		
			/* Create the directory.
			 */
			dir = folder_get_full_path(folder);
			path_create_directory(dir);
			
			/* See if it's there.
			 */
			if(!path_is_dir(dir))
			{
				g_free(dir);
				g_free(folder);
				return NULL;
			}
			
			g_free(dir);
		}

		folder->t.WindowTitle = g_strdup("");
		folder->t.Status = g_strdup("");
		folder->work = g_strdup("");
		
		list_init(&folder->Msgs);
		list_init(&folder->MsgQueue);
		
		pthread_mutex_init(&folder->Mutex, NULL);
	}

	return folder;
}

void folder_free(Folder *folder, gboolean skipcleanup, gboolean delete_dir)
{
	if(!folder)
		return;

	folder_close(folder);

		
	
	g_free(folder->t.WindowTitle);
	g_free(folder->t.Status);
	g_free(folder->work);

	g_free(folder);
}

gboolean folder_open(Folder *folder)
{
	glong count = 0;
	gboolean got_index;

	if(!folder)
		return FALSE;

	if(!folder_verify_directory(folder))
		return FALSE;
	
	/* Load the folder's index.
	 */
	got_index = folder_get_index(folder);

	/* If this is the incoming folder, check for 
	 * mail in the spool file, and read it.
	 */
	if(folder->Data.Type & FTF_INCOMING)
	{
		gchar *spool_file = g_strdup_printf("/var/spool/mail/%s", g_get_user_name());
		count = folder_import_msgs(folder, spool_file, TRUE);
		g_free(spool_file);
	}
	
	/* Check for mail in a folder-specific
	 * spool.
	 */
	count += folder_import_msgs(folder, folder->Data.SpoolFile, TRUE);
	
	/* Re-read or build index, if necessary.
	 */
	if(!got_index)
		folder_build_index(folder);
	else if(count)
		folder_update_index(folder);
	
	/* Open the window.
	 */
	folder_display_contents(folder);

	return TRUE;
}

void folder_close(Folder *folder)
{
	/* Need to check if the folder is
	 * being displayed, and if so,
	 * switch over to another folder,
	 * so we can close this one.
	 */
}

glong folder_gather_info(Folder *folder, gboolean from_disk)
{
	glong error;
	gchar *dir;
				
	g_return_val_if_fail(folder != NULL, -1);

	dir = folder_get_full_path(folder);

	folder->Data.MsgCount    =
	folder->Data.NewMsgs	=
	folder->Data.UnreadMsgs  =
	folder->Data.NotifyCount =
	folder->Data.Size   	= 0;

	if(!from_disk)
	{
		g_list_foreach(folder->Msgs.List, folder_count_item_cb, folder);
	}
	else
	{
		struct MatchState *ap;

		if((ap = g_malloc0(sizeof(struct MatchState))))
		{
			error = match_first(ap, dir, select_new_msg_file);
			while(!error)
			{
				struct dirent *de = ap->entry;
				struct stat *st = ap->st;

				folder->Data.MsgCount++;
				folder->Data.Size += st->st_size;

#if 0
				if(strstr(ap->ap_Info.fib_Comment, "Unr;"))
					folder->Data.UnreadMsgs++;
				else if(!strstr(ap->ap_Info.fib_Comment, "Opn;"))
					folder->Data.NewMsgs++;

				if(strstr(ap->ap_Info.fib_Comment, "Not;"))
					folder->Data.NotifyCount++;
#endif

				error = match_next(ap);
			}
			match_end(ap);

			g_free(ap);
		}
	}

	g_free(dir);
	
	return error;
}

/* Callback function for folder_gather_info().
 */
static void folder_count_item_cb(gpointer msgd, gpointer folderd)
{
	Folder *folder = (Folder *)folderd;
	MailMsg *msg = (MailMsg *)msgd;

	folder->Data.MsgCount++;
	folder->Data.Size += msg->Index.Size;
	if(msg->Index.Flags & MIEF_NEW)
		folder->Data.NewMsgs++;
	if(msg->Index.Flags & MIEF_UNREAD)
		folder->Data.UnreadMsgs++;
	if(msg->Index.Flags & MIEF_NOTIFY)
		folder->Data.NotifyCount++;
}

void folder_merge_info(struct FolderData *fd)
{
	Folder *folder;
	GList *node;

	for(node = g_list_first(euc.Folders.List); node; node = g_list_next(node))
	{
		folder = (Folder *)node->data;
	
		if(((fd->Type & FTF_SPECIAL) &&
		    (fd->Type == folder->Data.Type)))
		{
			memcpy(&folder->Data, fd, sizeof(struct FolderData));
		}
	}
}

gboolean folder_get_index(Folder *folder)
{
	MailIndexEntry mie;
	glong error = 0, count = 0;
	gchar *fname, *dir;
	FILE *fp;
	gboolean valid = FALSE, index_loaded = FALSE;

	g_return_val_if_fail(folder != NULL, -1);

	folder_set_status(folder, _("Reading index..."));
//	BusyPointer(folder->Window, TRUE);

	dir = folder_get_full_path(folder);
	fname = path_build(dir, INDEX_FILE, NULL);
	if(fp = fopen(fname, "r"))
	{
		MailMsg *msg;
		struct VersInfo vers;

		/* Read index */
		if(sizeof(struct VersInfo) == fread(&vers, 1, sizeof(struct VersInfo), fp))
		{
			if(vers.VersID == FOLDERINDEXVERSION)
			{
				glong seekp = -1, count = 0;
				gboolean rdone = FALSE;

				while((-1 != (seekp = fseek(fp, 0, SEEK_CUR))) &&
					 fread(&mie, sizeof(MailIndexEntry), 1, fp))
				{
					mie.Position = seekp;

					if(mie.Flags & (MIEF_OLD | MIEF_PURGED))
					{
#if 0
						if(mie.Flags & MIEF_PURGED)
						{
							/* Delete msg files for purged messages.
							 */
							unlink(mie.Filename);
						}
#endif

						continue;
					}

					if(msg = mailmsg_new(folder))
					{
						memcpy(&msg->Index, &mie, sizeof(MailIndexEntry));
						
						if(!mailmsg_find_by_filename(folder, msg->Index.Filename))
						{
							mailmsg_update_info(msg, FALSE);
							list_append(&folder->Msgs, msg);
							
							count++;
						}
						else
							mailmsg_free(msg);
					}
					else
					{
						rdone = TRUE;
						break;
					}
				}
			}
			else
			{
				if(vers.VersID < FOLDERINDEXVERSION)
					error = err_VERSIONTOOOLD;
				else
					error = err_VERSIONTOONEW;
			}
		}
		else
			error = err_CORRUPTEDFOLDERINDEX;

		fclose(fp);
	}
	
	if(count)
		index_loaded = TRUE;

	g_free(fname);
	g_free(dir);

	if(!valid)
		error = err_OUTOFDATE;

	folder_set_status(folder, NULL);
//	BusyPointer(folder->Window, FALSE);

	return index_loaded;
}

#if 0
gboolean folder_write_index(Folder *folder)
{
	glong error = 0, count = 0;
	gchar *fname, *new_index, *dir;
	FILE *fp;
	gboolean valid = FALSE, index_written = FALSE;

	g_return_val_if_fail(folder != NULL, FALSE);

	folder_set_status(folder, _("Writing index..."));
	busy_pointer(folder->w.Window, TRUE);

	dir = folder_get_full_path(folder);
	fname = path_build(dir, INDEX_FILE, NULL);
	new_index = g_strdup_printf("%s.tmp", fname);
	if(fp = fopen(new_index, "w"))
	{
		MailMsg *msg;
		struct VersInfo vers;

		vers.VersID = FOLDERINDEXVERSION;
		strncpy(vers.Vers, FOLDERINDEXSTRING, sizeof(vers.Vers) - 1);

		/* Write index */
		if(sizeof(struct VersInfo) == fwrite(&vers, 1, sizeof(struct VersInfo), fp))
		{
			 GList *node;
			 gboolean error = FALSE;

			 for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
			 {
				    gchar *mfile;
				    gboolean exists = FALSE;

				    msg = (MailMsg *)node->data;

				    /* Check if the file exists.
					*/
				    mfile = path_build(dir, msg->Index.Filename, NULL);
				    exists = path_is_file(mfile);
				    g_free(mfile);
				    if(!exists)
						  continue;

				    /* Check for OLD or PURGED flags.
					*/
				    if(msg->Index.Flags & (MIEF_OLD | MIEF_PURGED))
				    {
						if(msg->Index.Flags & MIEF_PURGED)
						{
							mfile = path_build(dir, msg->Index.Filename, NULL);
							unlink(mfile);
							g_free(mfile);
						}

						continue;
				    }

				    /* Write the entry.
				     */
				    if(sizeof(MailIndexEntry) != fwrite(&msg->Index, 1, sizeof(MailIndexEntry), fp))
				    {
						  error = TRUE;
						  break;
				    }
			}

			if(!error)
			    index_written = TRUE;
		}

		fclose(fp);
	}

	if(index_written)
	{
		   unlink(fname);
		   rename(new_index, fname);
	}
	
	g_free(new_index);
	g_free(fname);
	g_free(dir);

	folder_set_status(folder, NULL);
	busy_pointer(folder->w.Window, FALSE);

	return index_written;
}
#endif

gboolean folder_write_data(Folder *folder, gboolean force)
{
	gchar *fname;
	gboolean success = TRUE;
	int fd;
	FILE *fdp = NULL;

	g_return_val_if_fail(folder != NULL, FALSE);
	
	if(!(folder->Data.Flags & FFF_UPDATE) &&
	   !force)
		return TRUE;
		
	fname = folder_get_file_path(folder, FOLDERDATA_FILE);

	if(fdp = fopen(fname, "w"))
	{
		folder->Data.Flags &= ~(FFF_ALREADYCLEANED | FFF_UPDATE);

		fprintf(fdp, "Name %s\n", folder->Data.Name);
		fprintf(fdp, "BelongsTo %s\n", folder->Data.BelongsTo);
		fprintf(fdp, "Directory %s\n", folder->Data.Directory);
		fprintf(fdp, "SpoolFile %s\n", folder->Data.SpoolFile);
		fprintf(fdp, "Type %8.8lx\n", (glong)folder->Data.Type);
		fprintf(fdp, "Sort %8.8lx\n", (glong)folder->Data.SortBy);
		fprintf(fdp, "Flags %8.8lx\n", (glong)folder->Data.Flags);
		fprintf(fdp, "NextMsgNum %ld\n", (glong)folder->Data.NextMsgNum);
		fprintf(fdp, "MsgCount %ld,%ld,%ld,%ld\n", (glong)folder->Data.MsgCount, (glong)folder->Data.NewMsgs, (glong)folder->Data.UnreadMsgs, (glong)folder->Data.NotifyCount);
		fprintf(fdp, "MaxMsgs %ld\n", folder->Data.MaxMsgs);
		fprintf(fdp, "Size %ld\n", folder->Data.Size);
#ifdef EUCALYPTUS_PRO
		fprintf(fdp, "DefaultTo %s\n", folder->Data.DefaultTo);
#endif
		if(folder->Data.OpenAtStartup)
			fprintf(fdp, "OpenAtStartup\n");
		if(folder->Data.ApplyFiltersOnImport)
			fprintf(fdp, "ApplyFiltersOnImport\n");
		if(folder->Data.ProtectMsgs)
			fprintf(fdp, "ProtectMsgs\n");
		fprintf(fdp, "#\n");

		fflush(fdp);
		fclose(fdp);
	}
	else
		success = FALSE;

	g_free(fname);	

	return success;
}

void euc_sort_folders(void)
{
}

