/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: msg_gui.c
 *   	 GUI-related functions for messages
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#include "eucalyptus.h"
#include "gui_menus.h"
#include "gui_toolbars.h"

gboolean mailmsg_open_window(MailMsg *msg)
{
	GtkWidget *pane1hbox, *pane, *vbox, *hbox;
	GnomeUIInfo *menu, *toolbar;
	glong i;
	gboolean success = FALSE;

	g_return_val_if_fail(msg != NULL, 0);

	if(msg->Window)
		return TRUE;
	
	/* Some preliminary work...
	 * Get the message's parts.
	 */
	if(!mailmsg_get_parts(msg))
		return FALSE;
		
	/* Sort the headers according to user
	 * preference.
	 */
	mailmsg_sort_headers(msg);
	
	/* Create the main window.
	 */
	g_snprintf(msg->t.WindowTitle, sizeof(msg->t.WindowTitle), 
				 "%s (%s [%s])", 
				 msg->Index.Subject, 
				 msg->Index.From, 
				 msg->t.Date);

	msg->Window = gnome_app_new(APPNAME, msg->t.WindowTitle);

	/* Connect the signals.
	*/
	gtk_signal_connect(GTK_OBJECT(msg->Window), "delete_event", 
		   GTK_SIGNAL_FUNC(mailmsg_close_requested_cb), msg);
	gtk_signal_connect(GTK_OBJECT(msg->Window), "destroy", 
		   GTK_SIGNAL_FUNC(mailmsg_destroy_cb), msg);

	/* Add menus.
	*/
	gnome_app_create_menus_with_data(GNOME_APP(msg->Window), 
			    mailmsg_menu, msg);

	/* Add the toolbar.
	*/
	gnome_app_create_toolbar_with_data(GNOME_APP(msg->Window), 
			    mailmsg_toolbar, msg);

	/* Add the app bar.
	*/
	msg->StatusBar = gnome_appbar_new(FALSE, TRUE, 
			    GNOME_PREFERENCES_NEVER);
	gnome_app_set_statusbar(GNOME_APP(msg->Window), msg->StatusBar);

	/* Add other stuff to the window.
	*/
	msg->Layout = gtk_vbox_new(FALSE, 5);
	gtk_signal_connect(GTK_OBJECT(msg->Layout), "destroy",
			    GTK_SIGNAL_FUNC(mailmsg_layout_destroy_cb), msg);
	gnome_app_set_contents(GNOME_APP(msg->Window), msg->Layout);	

	/* Create a paned widget for the lists
	* and message display area.
	*/
	pane = gtk_vpaned_new();
	gtk_box_pack_start(GTK_BOX(msg->Layout), pane, 
			    TRUE, TRUE, GNOME_PAD_SMALL);
	gtk_paned_pack2(GTK_PANED(pane), msg->PaneLayout = gtk_vbox_new(FALSE, 0), TRUE, TRUE);

	/* A horizontal group for the header list
	* and part selector.
	*/
	pane1hbox = gtk_hpaned_new();
	gtk_paned_pack1(GTK_PANED(pane), pane1hbox, TRUE, TRUE);

	/* Create header list.
	*/
	msg->HeaderScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
	{
	   msg->HeaderList = gtk_clist_new(2);
	   for(i=0; i<2; i++)
		   gtk_clist_set_column_auto_resize(GTK_CLIST(msg->HeaderList), i, TRUE);
	   gtk_clist_set_column_justification(GTK_CLIST(msg->HeaderList), 0, GTK_JUSTIFY_RIGHT);
	   
	   /* Add to scrolled window.
	    */
	   gtk_container_add(GTK_CONTAINER(msg->HeaderScrolledWindow), msg->HeaderList);
	   
	   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(msg->HeaderScrolledWindow),
			   GTK_POLICY_AUTOMATIC,	/* horiztonal */
			   GTK_POLICY_AUTOMATIC);	/* vertical */
	}
	gtk_paned_pack1(GTK_PANED(pane1hbox), msg->HeaderScrolledWindow, TRUE, TRUE);

	mailmsg_populate_header_list(msg);

	/* Create the part selection area.
	*/
	vbox = gtk_vbox_new(FALSE, 5);
	{
	   msg->PartsScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
	   {
		   msg->PartsList = gtk_clist_new(2);
		   for(i=0; i<2; i++)
			   gtk_clist_set_column_auto_resize(GTK_CLIST(msg->PartsList), i, TRUE);
		   
		   /* Set signals.
		    */
		   gtk_signal_connect(GTK_OBJECT(msg->PartsList), "select-row",
				   GTK_SIGNAL_FUNC(mailmsg_select_part_cb), msg);

		   /* Add to scrolled window.
		    */
		   gtk_container_add(GTK_CONTAINER(msg->PartsScrolledWindow), msg->PartsList);
		   
		   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(msg->PartsScrolledWindow),
				   GTK_POLICY_AUTOMATIC,	/* horizontal */
				   GTK_POLICY_AUTOMATIC);	/* vertical */
	   }
	   gtk_box_pack_start(GTK_BOX(vbox), msg->PartsScrolledWindow, TRUE, TRUE, GNOME_PAD_SMALL);

	   msg->PartInfo = gtk_button_new_with_label(_("Info"));
	   gtk_box_pack_start(GTK_BOX(vbox), msg->PartInfo, FALSE, TRUE, GNOME_PAD_SMALL);
	}
	gtk_paned_pack2(GTK_PANED(pane1hbox), vbox, FALSE, TRUE);

	mailmsg_populate_parts_list(msg);

	if(prefs.MsgWindowLeft)
		gtk_widget_set_uposition(msg->Window,
				prefs.MsgWindowLeft,
				prefs.MsgWindowTop);
	if(prefs.MsgWindowWidth)
		gtk_widget_set_usize(msg->Window,
				prefs.MsgWindowWidth,
				prefs.MsgWindowHeight);
	else
		gtk_widget_set_usize(msg->Window,
				DEFAULT_MSG_WINDOW_WIDTH,
				DEFAULT_MSG_WINDOW_HEIGHT);

	/* Show it.
	 */
	gtk_widget_show_all(msg->Window);

	/* Some final detail work.
	 */
	gtk_widget_set_sensitive(msg->HeaderList, FALSE);

	msg->CurrentPart = 0;

	return TRUE;
}

void mailmsg_close_window(MailMsg *msg)
{
	/* Destroy the window.
	 */
	gtk_widget_destroy(msg->Window);
	
	/* Free the header list.
	 */
	mailmsg_free_header(msg->Headers);
	msg->Headers = NULL;

	/* Free the part list.
	 */
	mailmsg_free_parts(msg);
	 
	/* Clear the variables.
	 */
	msg->Window = NULL;
	msg->Layout = NULL;
	msg->Menu = NULL;
	msg->Toolbar = NULL;
	msg->StatusBar = NULL;
	msg->HeaderList = NULL;
	msg->PartsList = NULL;
	msg->PaneLayout = NULL;
	msg->DisplayLayout = NULL;
	msg->Display = NULL;
}

void mailmsg_adjust_menu(MailMsg *msg)
{
}

void mailmsg_update_status(MailMsg *msg)
{
}

gboolean mailmsg_populate_header_list(MailMsg *msg)
{
	GList *node;
	gboolean success = FALSE;
	
	if(!msg ||
	   !msg->HeaderList)
		return FALSE;

	/* Stop the widget from updating and
	 * clear its list.
	 */
	gtk_clist_freeze(GTK_CLIST(msg->HeaderList));
	gtk_clist_clear(GTK_CLIST(msg->HeaderList));

	/* Walk the list.
	 */	
	for(node = g_list_first(msg->Headers->List); node; node = g_list_next(node))
	{
		HdrNode *hnode;
		gchar *hdata;
		
		/* Get the header string.
		 */
		hnode = (HdrNode *)node->data;
		
		/* Clear out unnecessary information.
		 */
		header_remove_whitespace(hnode->Data, 0);
		header_decode(hnode->Data);

		/* See if we can split the header.
		 */
		if(strlen(hnode->Data) > 75)
		{
			glong l, oc = 0;
			gchar *p, *o;
			gboolean done = FALSE, firstline = TRUE;

			p = hnode->Data;

			while(!done)
			{
				oc = 0;
				o = p;

				if((l = strlen(p)) > 75)
				{
					p += 75;
					while((l > 0) &&
						 ((*p != ' ') &&
						  (*p != ',')))
					{
						p--;
						l--;
					}

					if(l)
					{
						if(*p == ',')
							p++;

						oc = *p;
						*p = '\0';
					}
					else
					{
						done = TRUE;
					}
				}
				else
					done = TRUE;

				if(l)
				{
					gchar *cols[2];
					
					cols[0] = firstline ? hnode->Tag : "";
					cols[1] = o;
					
					gtk_clist_append(GTK_CLIST(msg->HeaderList), cols);
				}

				if(oc)
				{
					*p = oc;
					if(*p != ',')
						p++;
				}

				firstline = FALSE;
			}
		}
		else
		/* No split necessary.
		 */
		{
			gchar *cols[2];

			/* Set the columns.
			 */			
			cols[0] = hnode->Tag;
			cols[1] = hnode->Data;
			
			/* Add the data.
			 */
			gtk_clist_append(GTK_CLIST(msg->HeaderList), cols);
		}
	}
	
	/* Resume updating.
	 */
	gtk_clist_thaw(GTK_CLIST(msg->HeaderList));

	return success;
}

gboolean mailmsg_populate_parts_list(MailMsg *msg)
{
	GList *node;
	gboolean success = FALSE;
	
	if(!msg ||
	   !msg->PartsList)
	   	return FALSE;

	/* Freeze and clear the list widget.
	 */
	gtk_clist_freeze(GTK_CLIST(msg->PartsList));
	gtk_clist_clear(GTK_CLIST(msg->PartsList));

	/* Walk the list.
	 */	
	node = g_list_first(msg->Parts.List);
	while(node)
	{
		MsgPart *part = (MsgPart *)node->data;
		gchar *cols[2];

		/* Set up the columns.
		 */		
		cols[0] = ""; // FIXME: need an image here
		cols[1] = part->Name;
	
		/* Add the node to the widget.
		 */
		gtk_clist_append(GTK_CLIST(msg->PartsList), cols);
	
		node = g_list_next(node);
	}

	/* Resume updating.
	 */	
	gtk_clist_thaw(GTK_CLIST(msg->PartsList));
	
	return success;
}

#if 0
LONG mailmsg_open_set_status(MailMsg *msg)
{
     struct Window *window;
     struct Gadget *layout, *new, *unread, *replied, *forwarded, *bounced, *deleted, *dontdelete, *notify;
     Object *winobj;
     LONG error = 0, flags = 0;
     BOOL done = FALSE;

     if(!msg ||
        !msg)
          return(0);

     flags = msg->Index.Flags;

     if(!(winobj = NewObject(WINDOW_GetClass(), NULL,
               WINDOW_Position,         WPOS_CENTERSCREEN,

               WA_Activate,             TRUE,
               WA_CloseGadget,          TRUE,
               WA_DragBar,              TRUE,
               WA_DepthGadget,          TRUE,
               WA_RMBTrap,              TRUE,
               WA_ReportMouse,          TRUE,
               WINDOW_BackFillName,     sprefs->Backfill,

               WA_IDCMP,                IDCMP_RAWKEY,

               WA_PubScreen,            euc.Screen,
               WA_Title,                LocaleString(ETXT_TITLE_MESSAGESTATUS),
               WA_ScreenTitle,          eucsctitle,

               WINDOW_Layout,           layout = VGroupObject,
                    LAYOUT_DeferLayout, TRUE,
                    LAYOUT_SpaceOuter,  TRUE,
                    StartMember,   VGroupObject,
                         LAYOUT_SpaceOuter,  TRUE,
                         LAYOUT_BevelStyle,       BVS_GROUP,
                         LAYOUT_Label,            LocaleString(ETXT_SETSTATUS_GROUP_FLAGS),
                         StartMember,        new = CheckBoxObject,
                              GA_Text,            LocaleString(ETXT_GAD_SETSTATUS_NEW),
                              GA_Selected,        flags & MIEF_NEW,
                              GA_ID,              GAD_SETSTATUS_NEW,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         StartMember,        unread = CheckBoxObject,
                              GA_Text,            LocaleString(ETXT_GAD_SETSTATUS_UNREAD),
                              GA_Selected,        flags & MIEF_UNREAD,
                              GA_ID,              GAD_SETSTATUS_UNREAD,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         StartMember,        replied = CheckBoxObject,
                              GA_Text,            LocaleString(ETXT_GAD_SETSTATUS_REPLIED),
                              GA_Selected,        flags & MIEF_REPLIED,
                              GA_ID,              GAD_SETSTATUS_REPLIED,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         StartMember,        forwarded = CheckBoxObject,
                              GA_Text,            LocaleString(ETXT_GAD_SETSTATUS_FORWARDED),
                              GA_Selected,        flags & MIEF_FORWARDED,
                              GA_ID,              GAD_SETSTATUS_FORWARDED,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         StartMember,        bounced = CheckBoxObject,
                              GA_Text,            LocaleString(ETXT_GAD_SETSTATUS_BOUNCED),
                              GA_Selected,        flags & MIEF_BOUNCED,
                              GA_ID,              GAD_SETSTATUS_BOUNCED,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         StartMember,        deleted = CheckBoxObject,
                              GA_Text,            LocaleString(ETXT_GAD_SETSTATUS_DELETED),
                              GA_Selected,        flags & MIEF_DELETED,
                              GA_ID,              GAD_SETSTATUS_DELETED,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         StartMember,        dontdelete = CheckBoxObject,
                              GA_Text,            LocaleString(ETXT_GAD_SETSTATUS_DONTDELETE),
                              GA_Selected,        flags & MIEF_DONTDELETE,
                              GA_ID,              GAD_SETSTATUS_DONTDELETE,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         StartMember,        notify = CheckBoxObject,
                              GA_Text,            LocaleString(ETXT_GAD_SETSTATUS_NOTIFY),
                              GA_Selected,        flags & MIEF_NOTIFY,
                              GA_ID,              GAD_SETSTATUS_NOTIFY,
                              GA_RelVerify,       TRUE,
                         EndMember,
                    EndMember,
                    StartMember,   ButtonObject,
                         GA_Text,       LocaleString(ETXT_GAD_SETSTATUS_OKAY),
                         GA_ID,              GAD_SETSTATUS_OKAY,
                         GA_RelVerify,       TRUE,
                    EndMember,
               End,
               TAG_END)))
          goto thatsit;

     if(!(window = CA_OpenWindow(winobj)))
          goto thatsit;

     while(!done)
     {
          LONG signals;

          signals = Wait((1L << window->UserPort->mp_SigBit) |
                         SIG_KILL);

          if(signals & SIG_KILL)
          {
               Signal(FindTask(NULL), SIG_KILL);
               break;
          }

          if(signals & (1L << window->UserPort->mp_SigBit))
          {
               LONG result, code = 0;

               while(WMHI_LASTMSG != (result = CA_HandleInput(winobj, code)))
               {
                    switch(result & WMHI_CLASSMASK)
                    {
                         case WMHI_CLOSEWINDOW:
                              done = TRUE;
                              break;
                         case WMHI_GADGETUP:
                              switch(result & WMHI_GADGETMASK)
                              {
                                   case GAD_SETSTATUS_NEW:
                                        if(flags & MIEF_NEW)
                                             flags &= ~MIEF_NEW;
                                        else
                                             flags |= MIEF_NEW;
                                        break;
                                   case GAD_SETSTATUS_UNREAD:
                                        if(flags & MIEF_UNREAD)
                                             flags &= ~MIEF_UNREAD;
                                        else
                                             flags |= MIEF_UNREAD;
                                        break;
                                   case GAD_SETSTATUS_REPLIED:
                                        if(flags & MIEF_REPLIED)
                                             flags &= ~MIEF_REPLIED;
                                        else
                                             flags |= MIEF_REPLIED;
                                        break;
                                   case GAD_SETSTATUS_FORWARDED:
                                        if(flags & MIEF_FORWARDED)
                                             flags &= ~MIEF_FORWARDED;
                                        else
                                             flags |= MIEF_FORWARDED;
                                        break;
                                   case GAD_SETSTATUS_BOUNCED:
                                        if(flags & MIEF_BOUNCED)
                                             flags &= ~MIEF_BOUNCED;
                                        else
                                             flags |= MIEF_BOUNCED;
                                        break;
                                   case GAD_SETSTATUS_DELETED:
                                        if(flags & MIEF_DELETED)
                                             flags &= ~MIEF_DELETED;
                                        else
                                             flags |= MIEF_DELETED;
                                        break;
                                   case GAD_SETSTATUS_DONTDELETE:
                                        if(flags & MIEF_DONTDELETE)
                                             flags &= ~MIEF_DONTDELETE;
                                        else
                                             flags |= MIEF_DONTDELETE;
                                        break;
                                   case GAD_SETSTATUS_NOTIFY:
                                        if(flags & MIEF_NOTIFY)
                                             flags &= ~MIEF_NOTIFY;
                                        else
                                             flags |= MIEF_NOTIFY;
                                        break;
                                   case GAD_SETSTATUS_OKAY:
                                        msg->Index.Flags = flags;
                                        msg->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
                         SetFolderIndexValid(msg, FALSE);
                                        done = TRUE;
                                        break;
                              }
                              break;
                    }
               }
          }
     }

     thatsit:
     if(winobj)
          DisposeObject(winobj);

     return(error);
}

MsgPart *SelectMsgPart(MailMsg *msg)
{
     struct Window *window;
     Object *winobj;
     struct Gadget *layout, *browser;
     MsgPart *mp;
     struct List list;
     struct Node *node;
     LONG x;
     BOOL done = FALSE, abort = FALSE;

     if(!msg)
          return(NULL);

     NewList(&list);
     if(node = AllocListBrowserNode(1,
               LBNA_Column,        0,
                    LBNCA_Text,         LocaleString(ETXT_SELECTATTACHMENT_ALL),
               TAG_END))
     {
          AddTail(&list, node);
     }
     else
     {
          mp = NULL;
          goto thatsit;
     }
     for(mp = (MsgPart *)msg->Parts.lh_Head, x = 0;
         mp->Node.ln_Succ;
         mp = (MsgPart *)mp->Node.ln_Succ, x++)
     {
          if(!x)
               continue;

          if(node = AllocListBrowserNode(1,
                    LBNA_Column,        0,
                         LBNCA_Text,         mp->Name,
                    TAG_END))
          {
               AddTail(&list, node);
          }
          else
          {
               mp = NULL;
               goto thatsit;
          }
     }

     mp = NULL;

     if(!(winobj = NewObject(WINDOW_GetClass(), NULL,
               WINDOW_Position,         WPOS_CENTERMOUSE,
               WA_InnerWidth,           200,
               WA_InnerHeight,          150,

               WA_Activate,             TRUE,
               WA_CloseGadget,          TRUE,
               WA_DragBar,              TRUE,
               WA_DepthGadget,          TRUE,
               WA_SizeGadget,           TRUE,
               WA_SizeBBottom,          TRUE,
               WA_RMBTrap,              TRUE,
               WINDOW_BackFillName,     sprefs->Backfill,

               WA_PubScreen,            euc.Screen,
               WA_Title,                LocaleString(ETXT_TITLE_SELECTATTACHMENT),
               WA_ScreenTitle,          eucsctitle,

               WINDOW_Layout,           layout = VGroupObject,
                    LAYOUT_DeferLayout, TRUE,
                    LAYOUT_SpaceOuter,  TRUE,
                    StartMember,        browser = ListBrowserObject,
                         LISTBROWSER_Labels,      &list,
                         LISTBROWSER_ShowSelected,TRUE,
                         GA_ID,                   GAD_SELECTATTACHMENT_LISTBROWSER,
                         GA_RelVerify,            TRUE,
                    EndMember,
               End,
               TAG_END)))
          goto thatsit;

     // open window
     if(!(window = CA_OpenWindow(winobj)))
          goto thatsit;

     while(!done)
     {
          LONG signals;

          signals = Wait((1L << window->UserPort->mp_SigBit) |
                         SIG_KILL);

          if(signals & SIG_KILL)
          {
               Signal(FindTask(NULL), SIG_KILL);
               abort = TRUE;
               break;
          }

          if(signals & (1L << window->UserPort->mp_SigBit))
          {
               LONG result, code = 0;

               while(WMHI_LASTMSG != (result = CA_HandleInput(winobj, code)))
               {
                    LONG flags, val;

                    switch(result & WMHI_CLASSMASK)
                    {
                         case WMHI_CLOSEWINDOW:
                              done = TRUE;
                              break;
                         case WMHI_GADGETUP:
                              switch(result & WMHI_GADGETMASK)
                              {
                                   case GAD_SELECTATTACHMENT_LISTBROWSER:
                                        GetAttr(LISTBROWSER_Selected, browser, (ULONG *)&val);
                                        if(val == 0)
                                             mp = (MsgPart *)MSGPART_ALL;
                                        else
                                             mp = (MsgPart *)GetListNum(&msg->Parts, val);

                                        GetAttr(LISTBROWSER_RelEvent, browser, (ULONG *)&flags);
                                        if(flags & LBRE_DOUBLECLICK)
                                             done = TRUE;
                                        break;
                              }
                              break;
                    }
               }
          }
     }

     thatsit:
     if(winobj)
          DisposeObject(winobj);

     while(node = RemHead(&list))
          FreeListBrowserNode(node);

     if(abort)
          mp = NULL;

     return(mp);
}

struct FlagNode *GetMultiMsgFlags(void)
{
     struct Window *window;
     struct Gadget *layout, *new, *unread, *replied, *forwarded, *bounced, *deleted, *dontdelete, *notify;
     struct List *new_list = NULL, *unread_list = NULL, *replied_list = NULL, *forwarded_list = NULL, *bounced_list = NULL, *deleted_list = NULL, *dontdelete_list = NULL, *notify_list = NULL;
     struct FlagNode *rfn, fn;
     Object *winobj;
     BOOL done = FALSE;

     if(!(new_list = ChooserLabels(LocaleString(ETXT_SETSTATUS_IGNORE), LocaleString(ETXT_SETSTATUS_SET), LocaleString(ETXT_SETSTATUS_CLEAR), NULL)))
          goto thatsit;
     if(!(unread_list = ChooserLabels(LocaleString(ETXT_SETSTATUS_IGNORE), LocaleString(ETXT_SETSTATUS_SET), LocaleString(ETXT_SETSTATUS_CLEAR), NULL)))
          goto thatsit;
     if(!(replied_list = ChooserLabels(LocaleString(ETXT_SETSTATUS_IGNORE), LocaleString(ETXT_SETSTATUS_SET), LocaleString(ETXT_SETSTATUS_CLEAR), NULL)))
          goto thatsit;
     if(!(forwarded_list = ChooserLabels(LocaleString(ETXT_SETSTATUS_IGNORE), LocaleString(ETXT_SETSTATUS_SET), LocaleString(ETXT_SETSTATUS_CLEAR), NULL)))
          goto thatsit;
     if(!(bounced_list = ChooserLabels(LocaleString(ETXT_SETSTATUS_IGNORE), LocaleString(ETXT_SETSTATUS_SET), LocaleString(ETXT_SETSTATUS_CLEAR), NULL)))
          goto thatsit;
     if(!(deleted_list = ChooserLabels(LocaleString(ETXT_SETSTATUS_IGNORE), LocaleString(ETXT_SETSTATUS_SET), LocaleString(ETXT_SETSTATUS_CLEAR), NULL)))
          goto thatsit;
     if(!(dontdelete_list = ChooserLabels(LocaleString(ETXT_SETSTATUS_IGNORE), LocaleString(ETXT_SETSTATUS_SET), LocaleString(ETXT_SETSTATUS_CLEAR), NULL)))
          goto thatsit;
     if(!(notify_list = ChooserLabels(LocaleString(ETXT_SETSTATUS_IGNORE), LocaleString(ETXT_SETSTATUS_SET), LocaleString(ETXT_SETSTATUS_CLEAR), NULL)))
          goto thatsit;

     if(!(winobj = NewObject(WINDOW_GetClass(), NULL,
               WINDOW_Position,         WPOS_CENTERSCREEN,

               WA_Activate,             TRUE,
               WA_CloseGadget,          TRUE,
               WA_DragBar,              TRUE,
               WA_DepthGadget,          TRUE,
               WA_RMBTrap,              TRUE,
               WA_ReportMouse,          TRUE,
               WINDOW_BackFillName,     sprefs->Backfill,

               WA_IDCMP,                IDCMP_RAWKEY,

               WA_PubScreen,            euc.Screen,
               WA_Title,                LocaleString(ETXT_TITLE_MESSAGESTATUS),
               WA_ScreenTitle,          eucsctitle,

               WINDOW_Layout,           layout = VGroupObject,
                    LAYOUT_DeferLayout, TRUE,
                    LAYOUT_SpaceOuter,  TRUE,
                    StartMember,   VGroupObject,
                         LAYOUT_SpaceOuter,  TRUE,
                         LAYOUT_BevelStyle,       BVS_GROUP,
                         LAYOUT_Label,            LocaleString(ETXT_SETSTATUS_GROUP_FLAGS),
                         StartMember,        new = PopUpObject,
                              CHOOSER_Labels,     new_list,
                              CHOOSER_AutoFit,    TRUE,
                              GA_ID,              GAD_SETSTATUS_NEW,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_SETSTATUS_NEW)),
                         StartMember,        unread = PopUpObject,
                              CHOOSER_Labels,     unread_list,
                              CHOOSER_AutoFit,    TRUE,
                              GA_ID,              GAD_SETSTATUS_UNREAD,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_SETSTATUS_UNREAD)),
                         StartMember,        replied = PopUpObject,
                              CHOOSER_Labels,     replied_list,
                              CHOOSER_AutoFit,    TRUE,
                              GA_ID,              GAD_SETSTATUS_REPLIED,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_SETSTATUS_REPLIED)),
                         StartMember,        forwarded = PopUpObject,
                              CHOOSER_Labels,     forwarded_list,
                              CHOOSER_AutoFit,    TRUE,
                              GA_ID,              GAD_SETSTATUS_FORWARDED,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_SETSTATUS_FORWARDED)),
                         StartMember,        bounced = PopUpObject,
                              CHOOSER_Labels,     bounced_list,
                              CHOOSER_AutoFit,    TRUE,
                              GA_ID,              GAD_SETSTATUS_BOUNCED,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_SETSTATUS_BOUNCED)),
                         StartMember,        deleted = PopUpObject,
                              CHOOSER_Labels,     deleted_list,
                              CHOOSER_AutoFit,    TRUE,
                              GA_ID,              GAD_SETSTATUS_DELETED,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_SETSTATUS_DELETED)),
                         StartMember,        dontdelete = PopUpObject,
                              CHOOSER_Labels,     dontdelete_list,
                              CHOOSER_AutoFit,    TRUE,
                              GA_ID,              GAD_SETSTATUS_DONTDELETE,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_SETSTATUS_DONTDELETE)),
                         StartMember,        notify = PopUpObject,
                              CHOOSER_Labels,     dontdelete_list,
                              CHOOSER_AutoFit,    TRUE,
                              GA_ID,              GAD_SETSTATUS_NOTIFY,
                              GA_RelVerify,       TRUE,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_SETSTATUS_NOTIFY)),
                    EndMember,
                    StartMember,   ButtonObject,
                         GA_Text,       LocaleString(ETXT_GAD_SETSTATUS_OKAY),
                         GA_ID,              GAD_SETSTATUS_OKAY,
                         GA_RelVerify,       TRUE,
                    EndMember,
               End,
               TAG_END)))
          goto thatsit;

     if(!(window = CA_OpenWindow(winobj)))
          goto thatsit;

     while(!done)
     {
          LONG signals;

          signals = Wait((1L << window->UserPort->mp_SigBit) |
                         SIG_KILL);

          if(signals & SIG_KILL)
          {
               Signal(FindTask(NULL), SIG_KILL);
               break;
          }

          if(signals & (1L << window->UserPort->mp_SigBit))
          {
               LONG result, code = 0;

               while(WMHI_LASTMSG != (result = CA_HandleInput(winobj, code)))
               {
                    switch(result & WMHI_CLASSMASK)
                    {
                         case WMHI_CLOSEWINDOW:
                              done = TRUE;
                              break;
                         case WMHI_GADGETUP:
                              switch(result & WMHI_GADGETMASK)
                              {
                                   LONG val;

                                   case GAD_SETSTATUS_NEW:
                                        GetAttr(CHOOSER_Active, new, (ULONG *)&val);
                                        fn.New = val;
                                        break;
                                   case GAD_SETSTATUS_UNREAD:
                                        GetAttr(CHOOSER_Active, unread, (ULONG *)&val);
                                        fn.Unread = val;
                                        break;
                                   case GAD_SETSTATUS_REPLIED:
                                        GetAttr(CHOOSER_Active, replied, (ULONG *)&val);
                                        fn.Replied = val;
                                        break;
                                   case GAD_SETSTATUS_FORWARDED:
                                        GetAttr(CHOOSER_Active, forwarded, (ULONG *)&val);
                                        fn.Forwarded = val;
                                        break;
                                   case GAD_SETSTATUS_BOUNCED:
                                        GetAttr(CHOOSER_Active, bounced, (ULONG *)&val);
                                        fn.Bounced = val;
                                        break;
                                   case GAD_SETSTATUS_DELETED:
                                        GetAttr(CHOOSER_Active, deleted, (ULONG *)&val);
                                        fn.Deleted = val;
                                        break;
                                   case GAD_SETSTATUS_DONTDELETE:
                                        GetAttr(CHOOSER_Active, dontdelete, (ULONG *)&val);
                                        fn.DontDelete = val;
                                        break;
                                   case GAD_SETSTATUS_NOTIFY:
                                        GetAttr(CHOOSER_Active, notify, (ULONG *)&val);
                                        fn.Notify = val;
                                        break;
                                   case GAD_SETSTATUS_OKAY:
                                        if(rfn = (struct FlagNode *)AllocVec(sizeof(struct FlagNode), MEMF_CLEAR))
                                        {
                                             *rfn = fn;
                                        }
                                        done = TRUE;
                                        break;
                              }
                              break;
                    }
               }
          }
     }

     thatsit:
     if(winobj)
          DisposeObject(winobj);

     if(new_list)
          FreeChooserLabels(new_list);
     if(unread_list)
          FreeChooserLabels(unread_list);
     if(replied_list)
          FreeChooserLabels(replied_list);
     if(forwarded_list)
          FreeChooserLabels(forwarded_list);
     if(bounced_list)
          FreeChooserLabels(bounced_list);
     if(deleted_list)
          FreeChooserLabels(deleted_list);

     return(rfn);
}
#endif


/* Callbacks.
 */
gint mailmsg_close_requested_cb(GtkWidget *window, GdkEventAny *e, gpointer data)
{
	 /* Destroy the window.
	  */
	 return FALSE;
}

void mailmsg_destroy_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	/* Do cleanup.
	 */
	mailmsg_close_window(msg);

	/* Reset some vars.
	 */
}

void mailmsg_layout_destroy_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	   msg->Layout = NULL;
	   msg->DisplayLayout = NULL;
	   msg->Display = NULL;
}

void mailmsg_close_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	mailmsg_close_window(msg);
}

void mailmsg_compose_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp;

	if(comp = compose_new())
	{
		list_append(&euc.Composes, comp);

		compose_open_window(comp);
	}
}

void mailmsg_reply_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	if(!mailmsg_create_reply(msg, MRT_REPLY, 0, NULL))
		euc_request(msg->Window,
				_("Eucalyptus error"),
				_("Unable to create reply message."),
				euc_ok_button, 0,
				EUCRF_ASYNC);
}

void mailmsg_reply_none_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	if(!mailmsg_create_reply(msg, MRT_REPLY, MRTT_NOTEXT, NULL))
		euc_request(msg->Window,
				_("Eucalyptus error"),
				_("Unable to create reply message."),
				euc_ok_button, 0,
				EUCRF_ASYNC);
}

void mailmsg_reply_all_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	if(!mailmsg_create_reply(msg, MRT_REPLY, MRTT_ALL, NULL))
		euc_request(msg->Window,
				_("Eucalyptus error"),
				_("Unable to create reply message."),
				euc_ok_button, 0,
				EUCRF_ASYNC);
}

void mailmsg_reply_all_none_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	if(!mailmsg_create_reply(msg, MRT_REPLY, MRTT_NOTEXT | MRTT_ALL, NULL))
		euc_request(msg->Window,
				_("Eucalyptus error"),
				_("Unable to create reply message."),
				euc_ok_button, 0,
				EUCRF_ASYNC);
}

void mailmsg_forward_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	if(!mailmsg_create_reply(msg, MRT_FORWARD, 0, NULL))
		euc_request(msg->Window,
				_("Eucalyptus error"),
				_("Unable to create reply message."),
				euc_ok_button, 0,
				EUCRF_ASYNC);
}

void mailmsg_forward_some_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	if(!mailmsg_create_reply(msg, MRT_FORWARD, MRTT_SOMEHEADERS, NULL))
		euc_request(msg->Window,
				_("Eucalyptus error"),
				_("Unable to create reply message."),
				euc_ok_button, 0,
				EUCRF_ASYNC);
}

void mailmsg_forward_all_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	if(!mailmsg_create_reply(msg, MRT_FORWARD, MRTT_ALLHEADERS, NULL))
		euc_request(msg->Window,
				_("Eucalyptus error"),
				_("Unable to create reply message."),
				euc_ok_button, 0,
				EUCRF_ASYNC);
}

void mailmsg_bounce_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_print_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_delete_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	msg->Index.Flags ^= MIEF_DELETED;

	folder_update_index(msg->Folder);
	folder_update_visual(msg->Folder);
}

void mailmsg_get_addr_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_status_read_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	msg->Index.Flags &= ~(MIEF_NEW | MIEF_UNREAD);

	folder_update_visual(msg->Folder);
	folder_update_index_entry(msg->Folder, msg);
}

void mailmsg_status_unread_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	msg->Index.Flags &= ~(MIEF_NEW);
	msg->Index.Flags |= MIEF_UNREAD;

	folder_update_visual(msg->Folder);
	folder_update_index_entry(msg->Folder, msg);
}

void mailmsg_status_important_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

	msg->Index.Flags |= MIEF_NOTIFY;

	folder_update_visual(msg->Folder);
	folder_update_index_entry(msg->Folder, msg);
}

void mailmsg_status_set_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_check_sig_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_decrypt_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_extract_key_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_save_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_save_attach_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_save_raw_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_move_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_copy_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_export_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_prev_msg_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_next_msg_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_uudecode_cb(GtkWidget *widget, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;

}

void mailmsg_select_part_cb(GtkCList *list, gint row, gint column, GdkEventButton *evnt, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;
	MsgPart *part;

	msg->CurrentPart = row;

	if(part = (MsgPart *)g_list_nth_data(msg->Parts.List, msg->CurrentPart))
	{
		errno = 0;

		if(!msgpart_display(msg, part) && errno)
		{
			gchar *emsg;
			
			euc_request(msg->Window,
					_("Eucalyptus error"),
					emsg = g_strdup_printf("%s:\n%s", _("Unable to display message part"), strerror(errno)),
					euc_ok_button, 0,
					0);
			g_free(emsg);
		}
	}
}

