/*
 * match.c
 * by Paul A. Schifferer
 *
 * File matching functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include "match.h"
#include "regexp.h"
#include "path.h"

int select_msg_file(const struct dirent *de)
{
	return !strncmp("msg", de->d_name, 3);
}

int select_new_msg_file(const struct dirent *de)
{
	int is_match = FALSE;

	if(!strncmp(de->d_name, "msg", 3) ||
	   !strncmp(de->d_name, "new", 3))
	   	is_match = TRUE;

	return is_match;
}

int select_addr_file(const struct dirent *de)
{
	return !strncmp("adr", de->d_name, 3);
}

glong match_first(struct MatchState *ms, gchar *dir, int (*func)(const struct dirent *))
{
	gchar *dirname = dir;

	g_return_val_if_fail(ms != NULL, -1);	
	
	if(!dir)
		dirname = ".";
	
	errno = 0;
	
	if(ms)
	{
		ms->dirname = g_strdup(dirname);
		
		if(!func)
			func = select_msg_file;
			
		if((int)func < MATCH_FUNC_MAX)
		{
			switch((int)func)
			{
				case MATCH_FUNC_MSG:
					func = select_msg_file;
					break;
					
				case MATCH_FUNC_NEW_MSG:
					func = select_new_msg_file;
					break;
					
				case MATCH_FUNC_ADR:
					func = select_addr_file;
					break;
			}
		}
		
		ms->count = scandir(dirname, &ms->dirents, func, alphasort);

		switch(ms->count)
		{
			gchar *fname;

			case -1:
			case 0:
				errno = ENOENT;
				break;
				
			default:
				ms->entry = ms->dirents[ms->num = 0];
				fname = path_build(dirname, ms->entry->d_name, NULL);
				ms->st = path_stat(fname);
				g_free(fname);
				break;
		}
	}
	else
		errno = ENOMEM;
	
	return errno;
}

glong match_next(struct MatchState *ms)
{
	g_return_val_if_fail(ms != NULL, -1);

	errno = 0;

	if(++ms->num < ms->count)
	{
		gchar *fname;

		if(ms->st)
			g_free(ms->st);
			
		ms->entry = ms->dirents[ms->num];
		fname = path_build(ms->dirname, ms->entry->d_name, NULL);
		ms->st = path_stat(fname);
		g_free(fname);
	}
	else
		errno = ENOENT;
	
	return errno;
}

void match_end(struct MatchState *ms)
{
	g_return_if_fail(ms != NULL);

	g_free(ms->dirname);
	g_free(ms->meta);
	free(ms->dirents);
	g_free(ms->st);
	
	errno = 0;
}
