/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: msg_display.c
 *   	 Display functions for messages
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef USE_GTKHTML
#include <gtkhtml/gtkhtml.h>
#endif
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gnome-canvas-pixbuf.h>

#include "eucalyptus.h"

gboolean msgpart_display(MailMsg *msg, MsgPart *part)
{
	GtkWidget *old_layout = NULL;
	Viewer *vnode;
	gint hpolicy, vpolicy;
	gchar *content_type = "text/plain", *displayfname = NULL;
	gboolean success = TRUE, use_scrw = TRUE;

	if(!msg ||
	   !part)
		return FALSE;

	/* Process the part.
	 */
	if(!msgpart_process(msg, part, (prefs.StripCodes ? 0 : SMPF_FORDISPLAY) | SMPF_CONVERTTABS))
	{
		return FALSE;
	}

	/* Look for the Content-Type: header
	 * for this part, so we know what we're
	 * trying to display.
	 */
	if(part->Headers)
	{
		HdrNode *hnode;

		if(hnode = header_find_tag("Content-Type", part->Headers, 0))
			content_type = hnode->Data;
	}

	/* Check if there's a viewer configured
	 * for this type.
	 */
	if(vnode = viewer_get_node(content_type))
	{
		gchar cmdline[WORKLEN], buf[3];

		/* If a preprocessor is specified, then run it through
		 * the preprocessor first; then pass on to viewer/datatype.
		 */
		if(*vnode->Preprocess)
		{
			glong x;
			gchar fileadded = 0, tempfile[FILENAME_MAX];

			// get filename
			strncpy(tempfile, g_get_tmp_dir(), sizeof(tempfile) - 1);
			strncat(tempfile, G_DIR_SEPARATOR_S "eucalyptusXXXXXX", sizeof(tempfile) - 1);
			displayfname = mktemp(tempfile);

			for(x=0; vnode->Preprocess[x]; x++)
			{
				switch(vnode->Preprocess[x])
				{
					gchar buf[5];

					case '%':
						switch(vnode->Preprocess[++x])
						{
							case '%':
								strncat(cmdline, "%", sizeof(cmdline) - 1);
								break;
							case 'i':
								strncat(cmdline, part->Filename, sizeof(cmdline) - 1);
								fileadded |= 1;
								break;
							case 'o':
								strncat(cmdline, displayfname, sizeof(cmdline) - 1);
								fileadded |= 2;
								break;
							default:
								sprintf(buf, "%c", vnode->Preprocess[x]);
								strncat(cmdline, buf, sizeof(cmdline) - 1);
								break;
						}
						break;
					default:
						sprintf(buf, "%c", vnode->Preprocess[x]);
						strncat(cmdline, buf, sizeof(cmdline) - 1);
						break;
				}
			}

			/* Check if input file was added.
			 */
			if(!(fileadded & 1))
			{
				strncat(cmdline, " ", sizeof(cmdline) - 1);
				strncat(cmdline, part->Filename, sizeof(cmdline) - 1);
			}
			/* Check if output file was added.
			 */
			if(!(fileadded & 2))
			{
				strncat(cmdline, " ", sizeof(cmdline) - 1);
				strncat(cmdline, displayfname, sizeof(cmdline) - 1);
			}

			busy_pointer(msg->Window, TRUE);

			/* Run the preprocess command.
			 */
			if(-1 == gnome_execute_shell(NULL, cmdline))
			{
				euc_request(msg->Window,
						_("Eucalyptus error"),
						_("Unable to execute preprocessor command."),
						euc_ok_button, 0,
						EUCRF_ASYNC);
				return FALSE;
			}

			busy_pointer(msg->Window, FALSE);
		}

		/* Now check if there is an action available for
		 * this type.  If not, drop through to internal
		 * display handling.
		 */
		if(*vnode->Action)
		{
			glong x;
			gboolean fileadded = FALSE;

			/* If "None" is the specified action for
			 * this type of data, then the user is
			 * explicitly requesting that nothing be
			 * displayed.
			 * FIXME: Should probably put some text in the
			 * window indicating this.
			 */
			if(!strcasecmp(vnode->Action, "None"))
			{
				return FALSE;
			}

			for(x=0; vnode->Action[x]; x++)
			{
				switch(vnode->Action[x])
				{
					case '%':
						switch(vnode->Action[++x])
						{
							case '%':
								strncat(cmdline, "%", sizeof(cmdline) - 1);
								break;
							case 'f':
								if(strlen(displayfname))
									strncat(cmdline, displayfname, sizeof(cmdline) - 1);
								else
									strncat(cmdline, part->Filename, sizeof(cmdline) - 1);
								fileadded = TRUE;
								break;
							default:
								sprintf(buf, "%%%c", vnode->Action[x]);
								strncat(cmdline, buf, sizeof(cmdline) - 1);
								break;
						}
						break;
					default:
						sprintf(buf, "%c", vnode->Action[x]);
						strncat(cmdline, buf, sizeof(cmdline) - 1);
						break;
				}
			}

			/* Check if data file was added.
			 */
			if(!fileadded)
			{
				strncat(cmdline, " ", sizeof(cmdline) - 1);
				if(strlen(displayfname))
					strncat(cmdline, displayfname, sizeof(cmdline) - 1);
				else
					strncat(cmdline, part->Filename, sizeof(cmdline) - 1);
			}

			busy_pointer(msg->Window, TRUE);

			/* Run viewer command.
			 */
			if(-1 == gnome_execute_shell(NULL, cmdline))
			{
				euc_request(msg->Window,
						_("Eucalyptus error"),
						_("Unable to execute viewer command."),
						euc_ok_button, 0,
						EUCRF_ASYNC);
				return FALSE;
			}

			/* Let user know this is being externally
			 * viewed.
			 */
			{
				GdkFont *font;
				gchar *text;

				if(msg->DisplayLayout)
					gtk_container_remove(GTK_CONTAINER(msg->PaneLayout), msg->DisplayLayout);
				msg->DisplayLayout = NULL;

				text = g_strdup_printf(_("Eucalyptus has spawned an external viewer for this data."
							"\n\n"
							"Filename: %s\n"
							"MIME type: %s\n"
							"Command executed: %s\n"),
							part->Name,
							content_type,
							cmdline);
				msg->Display = gnome_less_new();
				msg->DisplayLayout = msg->Display;
				hpolicy = GTK_POLICY_NEVER;
				vpolicy = GTK_POLICY_NEVER;
				font = gdk_font_load(prefs.MsgFont);
			 	gnome_less_set_font(GNOME_LESS(msg->Display), font);
				gnome_less_show_string(GNOME_LESS(msg->Display), text);
				g_free(text);
			}

			busy_pointer(msg->Window, FALSE);

			goto display_it;
		}
	}

	old_layout = msg->DisplayLayout;
	msg->DisplayLayout = NULL;
	msg->Display = NULL;
	
	/* Set policy variables.
	 */
	hpolicy = GTK_POLICY_NEVER;
	vpolicy = GTK_POLICY_NEVER;

	/* Display the new data, according to type.
	 */
	switch(part->Type & CONTENTTYPE_MASK)
	{
		GdkPixbuf *pixbuf;
		GnomeCanvasItem *item;

		case CONTENTTYPE_MESSAGE:
			/* This should get processed again as another
			 * message, then dumped to the text display?
			 * FIXME
			 */

		case CONTENTTYPE_TEXT:
		default:
			switch(part->Type & CONTENTSUBTYPE_MASK)
			{
				GdkFont *font;
				gchar *url;

				case CONTENTSUBTYPE_TEXT_HTML:
#if USE_GTKHTML
					msg->DisplayLayout = gtk_scrolled_window_new(NULL, NULL);
					msg->Display = gtk_html_new(
							gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(layout)),
							gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(layout)));

					gtk_signal_connect(GTK_OBJECT(msg->Display), "title_changed",
							GTK_SIGNAL_FUNC(msgpart_html_title_changed), msg);
					gtk_signal_connect(GTK_OBJECT(msg->Display), "url_requested",
							GTK_SIGNAL_FUNC(msgpart_html_url_requested), msg);
					gtk_signal_connect(GTK_OBJECT(msg->Display), "load_done",
							GTK_SIGNAL_FUNC(msgpart_html_load_done), msg);
					
					url = g_strdup_printf("file://localhost%s", part->Filename);
					part->StreamHandle = gtk_html_begin(GTK_HTML(msg->Display), url);
					gtk_html_parse(GTK_HTML(msg->Display));

					hpolicy = GTK_POLICY_AUTOMATIC;
					vpolicy = GTK_POLICY_AUTOMATIC;

					g_free(url);
					break;
#endif

				case CONTENTSUBTYPE_TEXT_ENRICHED:
				default:
					msg->Display = gnome_less_new();
					msg->DisplayLayout = msg->Display;
					font = gdk_font_load(prefs.MsgFont);
	   				gnome_less_set_font(GNOME_LESS(msg->Display), font);
					gnome_less_show_file(GNOME_LESS(msg->Display), part->Filename);
					use_scrw = FALSE;
					break;
			}
			break;

		case CONTENTTYPE_IMAGE:
			gtk_widget_push_visual(gdk_rgb_get_visual());
			gtk_widget_push_colormap(gdk_rgb_get_cmap());

			msg->Display = gnome_canvas_new();

			gtk_widget_pop_colormap();
			gtk_widget_pop_visual();

			pixbuf = gdk_pixbuf_new_from_file(part->Filename);

			item = gnome_canvas_item_new(gnome_canvas_root(GNOME_CANVAS(msg->Display)),
					GNOME_TYPE_CANVAS_PIXBUF,
					"pixbuf", pixbuf,
					"x", 0,
					"y", 0,
					"width", (double)pixbuf->art_pixbuf->width,
					"height", (double)pixbuf->art_pixbuf->height,
					NULL);

			gnome_canvas_set_scroll_region(GNOME_CANVAS(msg->Display),
					0.0, 0.0,
					pixbuf->art_pixbuf->width,
					pixbuf->art_pixbuf->height);

			hpolicy = GTK_POLICY_AUTOMATIC;
			vpolicy = GTK_POLICY_AUTOMATIC;

			break;

		case CONTENTTYPE_APPLICATION:
			switch(part->Type & CONTENTSUBTYPE_MASK)
			{
				default:
				case CONTENTSUBTYPE_APPLICATION_OCTETSTREAM:
#if 0
					{
						FILE *fp_in, *fp_out;
						glong tries = 0;
						gchar *tmpfile;

						tmpfile = tempnam(g_get_tmp_dir(), "euc");

						fp_in = fopen(part->Filename, "r");
						fp_out = fopen(tmpfile, "w");

						if(fp_in && fp_out)
						{
							while(tries < 3)
							{
								// specially format raw binary data (addr:XXXXXXXX (x4) (16-bytes of ASCII)
								if((binary_column == 16) || nomoredata)
								{    // spit out what we have accumulated
									glong j, k;
									gchar buf[101], tmp[21];

									// clear the temporary output buffer
									memset(buf, 0, sizeof(buf));

									// address
									g_snprintf(buf, sizeof(buf), "%8.8lX:", binary_address);

									// binary data
									for(j=0; j<16; j++)
									{
										if(binary_data[j] == -1)
											break;

										if((j % 4) == 0)
											strncat(buf, " ", sizeof(buf) - 1);

										g_snprintf(tmp, sizeof(tmp), "%2.2X", binary_data[j]);
										strncat(buf, tmp, sizeof(buf) - 1);
									}

									for(k=j; k<16; k++)
									{
										if((k % 4) == 0)
											strncat(buf, " ", sizeof(buf) - 1);

										strncat(buf, "  ", sizeof(buf) - 1);
									}

									strncat(buf, " ", sizeof(buf) - 1);

									// ascii data
									for(j=0; j<16; j++)
									{
										if(binary_data[j] == -1)
											break;

										if(isprint(binary_data[j]) ||
										   binary_data[j] > 160)
											g_snprintf(tmp, sizeof(tmp), "%c", binary_data[j]);
										else
										{
											if(binary_data[j] < 32)
											{
	//                                                                              g_snprintf(tmp, sizeof(tmp), "\033[7m%c\033[27m", binary_data[j] + 64);
												g_snprintf(tmp, sizeof(tmp), "%c", binary_data[j] + 64);
											}
											else
												strcpy(tmp, ".");
										}

										strncat(buf, tmp, sizeof(buf) - 1);
									}

									strncat(buf, "\n", sizeof(buf) - 1);
									fprintf(tmp_fp, buf);

									// reset buffers
									for(j=0; j<16; j++)
										binary_data[j] = -1;
									binary_column = 0;
									binary_address += 16;
									break;
								}
								else
								{
									switch(tries)
									{
										case 0:
											binary_data[binary_column] = (gint16)((triplet & 0xFF0000) >> 16);
											break;
										case 1:
											if(equals < 2)
												binary_data[binary_column] = (gint16)((triplet & 0xFF00) >> 8);
											else
												nomoredata = TRUE;
											break;
										case 2:
											if(!equals)
												binary_data[binary_column] = (gint16)((triplet & 0xFF));
											else
												nomoredata = TRUE;
											break;
									}

									binary_column++;
								}

								tries++;
							}
						}

						fclose(fp_in);
						fclose(fp_out);

						msg->DisplayLayout = msg->Display = gnome_less_new();
						hpolicy = GTK_POLICY_NEVER;
						vpolicy = GTK_POLICY_NEVER;
						font = gdk_font_load(prefs.MsgFont);
						gnome_less_set_font(GNOME_LESS(msg->Display), font);
						gnome_less_show_file(GNOME_LESS(msg->Display), tmpfile);

						free(tmpfile);
					}
#endif
					break;

				case CONTENTSUBTYPE_APPLICATION_POSTSCRIPT:
					break;
			}
			break;

		case CONTENTTYPE_AUDIO:
			break;

		case CONTENTTYPE_VIDEO:
			break;
	}

	if(!msg->Display)
	{
		GdkFont *font;
		gchar *text;

		text = g_strdup_printf(_("Eucalyptus cannot find a viewer configured for"
					"this data, and is unable to display it using the internal"
					"viewer.\n\n"
					"Filename: %s\n"
					"MIME type: %s\n"),
					part->Name,
					content_type);
		msg->Display = gnome_less_new();
		msg->DisplayLayout = msg->Display;
		font = gdk_font_load(prefs.MsgFont);
	   	gnome_less_set_font(GNOME_LESS(msg->Display), font);
		gnome_less_show_string(GNOME_LESS(msg->Display), text);
		use_scrw = FALSE;
		g_free(text);
	}

	if(old_layout)
		gtk_container_remove(GTK_CONTAINER(msg->PaneLayout), old_layout);

display_it:
	/* Add the new display object
	 * to the window.
	 */	
	if(!msg->DisplayLayout)
	{
		msg->DisplayLayout = gtk_scrolled_window_new(NULL, NULL);
		use_scrw = TRUE;
	}

	if(msg->DisplayLayout != msg->Display)
		gtk_container_add(GTK_CONTAINER(msg->DisplayLayout), msg->Display);
	gtk_box_pack_start(GTK_BOX(msg->PaneLayout), msg->DisplayLayout, TRUE, TRUE, 0);

	if(use_scrw)
	{
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(msg->DisplayLayout),
				hpolicy,
				vpolicy);
	}
	gtk_widget_show_all(msg->DisplayLayout);

	return success;
}

#ifdef USE_GTKHTML
void msgpart_html_title_changed(GtkHTML *html, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;
	MsgPart *part = (MsgPart *)msg->CurrentPart;
}

void msgpart_html_url_requested(GtkHTML *html, const gchar *url, GtkHTMLStreamHandle handle, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;
	MsgPart *part = (MsgPart *)msg->CurrentPart;
}

void msgpart_html_load_done(GtkHTML *html, gpointer data)
{
	MailMsg *msg = (MailMsg *)data;
	MsgPart *part = (MsgPart *)msg->CurrentPart;
}
#endif

