/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: parsers.c
 *         Msg header parsing functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "eucalyptus.h"

static gchar days[7][4]    = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
static gchar months[12][4] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

#define MAKE_ID(c1, c2, c3, c4)	((c1 << 24) | (c2 << 16) | (c3 << 8) | (c4))

void parse_address(gchar *hdr, gchar *name, glong namelen, gchar *addr, glong addrlen, gboolean to)
{
     gchar *c, *e, text[2048];

     if(!hdr)
          return;

	strncpy(text, hdr, sizeof(text) - 1);

     if(name)
     	memset(name, 0, namelen);
     if(addr)
     	memset(addr, 0, addrlen);

     if(c = strchr(text, '<'))
     {
          /* Parse address line as "My Name <me@myplace.com>" */
          *c = '\0';
          if(name)
               strncpy(name, text, namelen - 1);

          if(e = strchr(c + 1, '>'))
          {
               *e = '\0';

               if(addr)
                    strncpy(addr, c + 1, addrlen - 1);
          }

          if(name &&
             !*name)
               strncpy(name, addr ? addr : (c + 1), namelen - 1);
     }
     else
     {
          /* Parse address line as "me@myplace.com (My Name)" */
          if(c = strchr(text, '('))
          {
               *c = '\0';

               if(e = strchr(c + 1, ')'))
               {
                    *e = '\0';

                    if(name)
                         strncpy(name, c + 1, namelen - 1);
               }
          }

          if(addr)
               strncpy(addr, text, addrlen - 1);
          if(name &&
             !*name)
               strncpy(name, text, namelen - 1);
     }

     if(name)
     {
          header_remove_quotes(name);
          header_remove_whitespace(name, 0);
          header_decode(name);
     }
     if(addr)
     {
          header_remove_quotes(addr);
          header_remove_whitespace(addr, 0);
          header_decode(addr);
     }
}

time_t parse_msg_date(gchar *date, struct tm *tm)
{
     int i;
     static gchar mdays[] = {31,28,31,30,31,30,31,31,30,31,30,31};

     if (!date)
          return 0;

     /* parse string like "Wed, 28 Jun 1995 20:12:23 (+0200)" */

     date = strip_leading_blanks(date);

     for ( i = 0 ; i < 7 ; i++ )
     {
          if (!strncasecmp(date, days[i], 3))
          {
               date += 3;
               break;
          }
     }

     if (i < 7 && *date != ',' && *date != ' ')
          i = 7, date -= 3;

     if (i == 7)
          goto asctimeformat; /* Not RFC compliant date. Maybe it's asctime()? */

     tm->tm_wday = i;

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_mday = strtol( date, &date, 10 );

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     for ( i = 0 ; i < 12 ; i++ )
     {
          if (!strncasecmp(date, months[i], 3))
          {
               date += 3;
               break;
          }
     }
     tm->tm_mon = i;

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_year = strtol( date, &date, 10 );
     if (tm->tm_year < 100)
          (tm->tm_year += 1900);
     if (tm->tm_year < 1960)
          (tm->tm_year += 100);

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_hour = strtol( date, &date, 10 );

     while ( *date && (*date == ':' || *date == ' ') )
          ++date;

     tm->tm_min = strtol( date, &date, 10 );

     if ( *date == ':' )
          tm->tm_sec = strtol( ++date, &date, 10 );
     else
          tm->tm_sec = 0;

     goto timezonecheck;

asctimeformat:

     /* parse string like "Nov 23 1995 08:45:23 PM". Does not fill tm->tm_wday */

     for ( i = 0 ; i < 12 ; i++ )
     {
          if (!strncasecmp(date, months[i], 3))
          {
               date += 3;
               break;
          }
     }

     if (i == 12)
          goto spotformat; /* yet another broken date format */

     tm->tm_mon = i+1;

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_mday = strtol( date, &date, 10 );

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_year = strtol( date, &date, 10 );
     if (tm->tm_year < 100)
          (tm->tm_year += 1900);
     if (tm->tm_year < 1960)
          (tm->tm_year += 100);

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_hour = strtol( date, &date, 10 );

     while ( *date && (*date == ':' || *date == ' ') )
          ++date;

     tm->tm_min = strtol( date, &date, 10 );

     while ( *date && (*date == ':' || *date == ' ') )
          ++date;

     tm->tm_sec = strtol( date, &date, 10 );

     while ( *date && (*date == ':' || *date == ' ') )
          ++date;

     if (!strncasecmp(date, "PM", 2))
          (tm->tm_hour += 12);

     goto timezonecheck;

spotformat:

     /* parse string like "13 Apr 1996 14:05:39" */
     /* apparently THOR uses a variant: "18 Jan 97 16:59:06 -0500" */

     tm->tm_mday = strtol( date, &date, 10 );

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     for ( i = 0 ; i < 12 ; i++ )
     {
          if (!strncasecmp(date, months[i], 3))
          {
               date += 3;
               break;
          }
     }

     if (i == 12)
          goto umsformat;

     tm->tm_mon = i+1;

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_year = strtol( date, &date, 10 );
     if (tm->tm_year < 100)
          (tm->tm_year += 1900);
     if (tm->tm_year < 1960)
          (tm->tm_year += 100);

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_hour = strtol( date, &date, 10 );

     while ( *date && (*date == ':' || *date == ' ') )
          ++date;

     tm->tm_min = strtol( date, &date, 10 );

     while ( *date && (*date == ':' || *date == ' ') )
          ++date;

     tm->tm_sec = strtol( date, &date, 10 );

     goto timezonecheck;

umsformat:

     /* parse string like "Friday 30-Aug-96 17:45:22" - from UMS */

     for ( i = 0 ; i < 7 ; i++ )
     {
          if (!strncasecmp(date, days[i], 3))
          {
               date += 3;
               break;
          }
     }

     if (i == 7)
          return 0; /* couldn't parse the date at all */

     tm->tm_wday = i;

     while ( *date && !(*date >= '0' && *date <= '9') )
          ++date;

     tm->tm_mday = strtol( date, &date, 10 );

     while ( *date && (*date == ' ' || *date == '-') )
          ++date;

     for ( i = 0 ; i < 12 ; i++ )
     {
          if (!strncasecmp(date, months[i], 3))
          {
               date += 3;
               break;
          }
     }

     tm->tm_mon = i+1;

     while ( *date && (*date == ' ' || *date == '-') )
          ++date;

     tm->tm_year = strtol( date, &date, 10 );
     if (tm->tm_year < 100)
          (tm->tm_year += 1900);
     if (tm->tm_year < 1960)
          (tm->tm_year += 100);

     while ( *date && (*date == ' ' || *date == ',') )
          ++date;

     tm->tm_hour = strtol( date, &date, 10 );

     while ( *date && (*date == ':' || *date == ' ') )
          ++date;

     tm->tm_min = strtol( date, &date, 10 );

     while ( *date && (*date == ':' || *date == ' ') )
          ++date;

     tm->tm_sec = strtol( date, &date, 10 );

timezonecheck:

     while ( *date && (*date == ' ' || *date == ',' || *date == '(') ) /*)*/
          ++date;

     if ( *date == '+' || *date == '-' )
     {
          i = strtol(date, NULL, 10);
          (tm->tm_min -= i%100);
          (tm->tm_hour -= i/100);
     }
     else
     {
          gulong zzz = 0;

          memcpy( (void *)&zzz, date, sizeof(zzz) );

          if (!(zzz & 0xfff) && (zzz & 0xf000) >= 'A' && (zzz & 0xf000) <= 'Z')
          {
               static gchar mz[] =
               {
                    -1,-2,-3,-4,-5,-6,-7,-8,-9,0,-10,-11,-12,
                    +1,+2,+3,+4,+5,+6,+7,+8,+9,+10,+11,+12,0
               };

               (tm->tm_hour -= mz[((zzz>>24) - 'A')]);
          }
          else switch (zzz)
          {
	          case MAKE_ID('E','S','T',0):
	               (tm->tm_hour += 5);
	               break;
	          case MAKE_ID('E','D','T',0):
	               (tm->tm_hour += 4);
	               break;
	          case MAKE_ID('C','S','T',0):
	               (tm->tm_hour += 6);
	               break;
	          case MAKE_ID('C','D','T',0):
	               (tm->tm_hour += 5);
	               break;
	          case MAKE_ID('M','S','T',0):
	               (tm->tm_hour += 7);
	               break;
	          case MAKE_ID('M','D','T',0):
	               (tm->tm_hour += 6);
	               break;
	          case MAKE_ID('P','S','T',0):
	               (tm->tm_hour += 8);
	               break;
	          case MAKE_ID('P','D','T',0):
	               (tm->tm_hour += 7);
	               break;
          }
     }

     /* fix time after zone conversion */
     if ((gint16)tm->tm_min < 0)
     {
          (tm->tm_min += 60);
          --tm->tm_hour;
     }
     else
     if (tm->tm_min > 59)
     {
          (tm->tm_min -= 60);
          ++tm->tm_hour;
     }
     if ((gint16)tm->tm_hour < 0)
     {
          (tm->tm_hour += 24);
          --tm->tm_mday;
          --tm->tm_wday;
     }
     else
     if (tm->tm_hour > 23)
     {
          (tm->tm_hour -= 24);
          ++tm->tm_mday;
          ++tm->tm_wday;
     }
     (tm->tm_wday = tm->tm_wday % 7);

     if (!(tm->tm_year%4) && (tm->tm_year%100) && !(tm->tm_year%400)) /* leap year */
          mdays[1] = 29;
     else
          mdays[1] = 28;

     if (tm->tm_mday < 1)
     {
          --tm->tm_mon;
          if (tm->tm_mon < 1)
          {
               --tm->tm_year;
               (tm->tm_mon += 12);
          }
          tm->tm_mday = mdays[tm->tm_mon-1];
     }
     else
     if (tm->tm_mday > mdays[tm->tm_mon-1])
     {
          ++tm->tm_mon;
          if (tm->tm_mon > 12)
          {
               ++tm->tm_year;
               (tm->tm_mon -= 12);
          }
          tm->tm_mday = 1;
     }

	tm->tm_isdst = -1;

     return mktime(tm);
}

void parse_convert_tabs(gchar *buf, glong buflen, gint16 flags)
{
}
