/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: path.c
 *   	 Path construction/parsing functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <glib.h>

#include "path.h"

glong path_add_part(gchar *buf, const gchar *new_part, gint buflen)
{
	glong l = strlen(buf);
	
	g_return_val_if_fail(buf != NULL, 0);
	g_return_val_if_fail(new_part != NULL, 0);
	g_return_val_if_fail(buflen > 0, 0);
	
	if(l >= buflen - 1)
		return l;
	
	if(buf[l - 1] != G_DIR_SEPARATOR)
		buf[l] = G_DIR_SEPARATOR;
	buf[l + 1] = 0;
	
	strncat(buf, new_part, buflen - 1);
	
	return strlen(buf);
}

gchar **path_parse(const gchar *path)
{
	glong i, j, count = 0;
	gchar **items, *p_orig, *p;
	
	g_return_val_if_fail(path != NULL, NULL);

	p = p_orig = g_strdup(path);
	
	while(*p)
		if(*p++ == G_DIR_SEPARATOR)
			count++;
			
	count += 2;
	
	items = g_malloc0(sizeof(gchar *) * count);
	
	for(p=p_orig, i=j=0; ; i++)
	{
		if(p_orig[i] == G_DIR_SEPARATOR || p_orig[i] == 0)
		{
			glong len = &p_orig[i] - p;
			
			items[j] = g_malloc0(len + 1);
			strncpy(items[j], p, len);
			
			if(p_orig[i] == 0)
				break;
				
			p = &p_orig[i + 1];
			j++;
		}
	}

	g_free(p_orig);
	
	return items; 
}

void path_free_items(gchar **items)
{
	glong i = 0;
	
	if(items)
	{
		for(i=0; items[i]; i++)
			g_free(items[i]);
			
		g_free(items);
	}
}

gchar *path_file_part(gchar *path)
{
	gchar *fpart;
	
	if(fpart = (gchar *)strrchr(path, G_DIR_SEPARATOR))
		return fpart + 1;
		
	return path;
}

gchar *path_dir_part(const gchar *path)
{
	glong len;
	gchar *c, *new_str;
	
	g_return_val_if_fail(path != NULL, NULL);

	if(c = (gchar *)strrchr(path, G_DIR_SEPARATOR))
		len = c - path + 1;
	else
		len = strlen(path) + 1;
		
	new_str = g_malloc0(len);
	strncpy(new_str, path, len);
	
	return new_str;
}

gchar *path_build(const gchar *part1, ...)
{
	gchar path[FILENAME_MAX], *item;
	va_list args;

	g_return_val_if_fail(part1 != NULL, NULL);
	
	va_start(args, part1);

	memset(path, 0, sizeof(path));
	strncpy(path, part1, sizeof(path) - 1);
	while(item = va_arg(args, gchar *))
	{
		strncat(path, G_DIR_SEPARATOR_S, sizeof(path) - 1);
		strncat(path, item, sizeof(path) - 1);
	}
	
	va_end(args);

	return g_strdup(path);
}

gchar *path_prune_part(const gchar *path)
{
	glong len;
	gchar *c, *work, *new_str;
	
	g_return_val_if_fail(path != NULL, NULL);

	work = g_strdup(path);
	
	if(work[strlen(work) - 1] == G_DIR_SEPARATOR)
		work[strlen(work) - 1] = 0;
	
	if(c = (gchar *)strrchr(work, G_DIR_SEPARATOR))
		len = c - work + 1;
	else
		len = strlen(work) + 1;
		
	new_str = g_malloc0(len);
	strncpy((gchar *)new_str, (gchar *)work, len - 1);
	
	g_free(work);
	
	return new_str;
}

gboolean path_create_directory(const gchar *path)
{
	gboolean success = TRUE;

	g_return_val_if_fail(path != NULL, FALSE);
	
	if(path_is_dir(path))
		return TRUE;
		
	if(path_is_file(path))
		return FALSE;

	if(mkdir(path, 0777))
	{	/* Failed. */
		gchar *part = path_prune_part(path);

		if(path_create_directory(part))
		{
			if(mkdir(path, 0777))
				success = FALSE;
		}
		else
			success = FALSE;
		
		g_free(part);
	}
	
	return success;
}

gboolean path_is_file(const gchar *path)
{
	struct stat *st;
	gboolean is_file = FALSE;
	
	g_return_val_if_fail(path != NULL, FALSE);
	
	if(st = path_stat(path))
	{
		if(S_ISREG(st->st_mode))
			is_file = TRUE;
	
		g_free(st);
	}
	
	return is_file;
}

gboolean path_is_dir(const gchar *path)
{
	struct stat *st;
	gboolean is_dir = FALSE;
	
	g_return_val_if_fail(path != NULL, FALSE);

	if(st = path_stat(path))
	{
		if(S_ISDIR(st->st_mode))
			is_dir = TRUE;
	
		g_free(st);
	}
	
	return is_dir;
}

struct stat *path_stat(const gchar *path)
{
	struct stat *st = NULL;
	
	g_return_val_if_fail(path != NULL, NULL);

	if(st = g_malloc0(sizeof(struct stat)))
	{
		if(stat(path, st))
		{
			g_free(st);
			st = NULL;
		}
	}
	
	return st;
}

gchar *path_metadata(const gchar *path)
{
	gchar *meta = NULL;



	return meta;
}

