/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefs.c
 *   	 General preferences routines for ASCII-type prefs
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <glib.h>

#include "eucalyptus.h"

struct PrefsControl prefs_control;

/* Enumeration for prefs types.
 */
enum
{
	PREFS_TYPE_END=0,
	PREFS_TYPE_BOOL,
	PREFS_TYPE_INT,
	PREFS_TYPE_STRING,
	PREFS_TYPE_CHOICE,
	PREFS_TYPE_COMPOSITE,
	PREFS_TYPE_ITER,
};

struct PrefsTable
{
	gint Type;
	gchar *Item;
	gpointer Storage;
	glong Size;
	gpointer Default;
	gchar **Choices;
	gboolean (*ParseFunc)(gchar *);
	gchar * (*SaveFunc)();
	glong Flags;
};

struct PrefsFile
{
	gchar *Filename;
	struct PrefsTable *Table;
	void (*InitFunc)(void);
};

/* Flags for PrefsTable.
 */
enum
{
	PTF_GETONLY = 1 << 0,
	PTF_SAVEONLY = 1 << 1,
	PTF_PRIVATE = 1 << 2,
	PTF_SKIPDEFAULT = 1 << 3,
};

/* Prototype declarations for prefs functions.
 */
gboolean parse_main_window_pos(gchar *);
gchar *save_main_window_pos(void);
gboolean parse_msg_window_pos(gchar *);
gchar *save_msg_window_pos(void);
gboolean parse_compose_window_pos(gchar *);
gchar *save_compose_window_pos(void);
gboolean parse_folder_window_pos(gchar *);
gchar *save_folder_window_pos(void);
gboolean parse_pop_window_pos(gchar *);
gchar *save_pop_window_pos(void);
gboolean parse_smtp_window_pos(gchar *);
gchar *save_smtp_window_pos(void);
gboolean parse_addr_book_cols(gchar *);
gchar *save_addr_book_cols(void);
gboolean parse_personal_pen(gchar *);
gchar *save_personal_pen(void);
gboolean parse_priority_pen(gchar *);
gchar *save_priority_pen(void);
gboolean parse_important_pen(gchar *);
gchar *save_important_pen(void);
gboolean parse_retriever(gchar *);
gchar *save_retriever(glong);

/* Choice arrays.
 */
gchar *addrbooksort_choices[] = { "FIRSTNAME", "LASTNAME", "ADDRESS", NULL, };
gchar *securityprog_choices[] = { "PGP2", "PGP5", "GPG", NULL, };
gchar *folderupdates_choices[] = { "INTERACTIVE", "BATCH", NULL, };
gchar *sortorder_choices[] = { "FIRST", "MIXED", "LAST", NULL, };
gchar *replydefault_choices[] = { "SENDERQUOTE", "SENDERNOQUOTE", "ALLQUOTE", "ALLNOQUOTE", NULL, };
gchar *forwarddefault_choices[] = { "NONE", "BRIEF", "FULL", NULL, };
gchar *smtptype_choices[] = { "INTERNAL", "COMMAND", NULL, };
gchar *windowtype_choices[] = { "FULL", "SUMMARY", NULL, };
gchar *windowstate_choices[] = { "CLOSED", "ICONIFIED", "OPEN", NULL, };
gchar *eoltype_choices[] = { "CRLF", "CRONLY", "LFONLY", NULL, };

struct PrefsTable general_prefs_table[] =
{
	{ PREFS_TYPE_INT, "TabWidth", &prefs.TabWidth, 0, (gpointer)5, NULL, NULL, 0, },
	{ PREFS_TYPE_CHOICE, "SecurityProgram", &prefs.SecurityProgram, 0, (gpointer)0, securityprog_choices, NULL, NULL, 0, },
	{ PREFS_TYPE_CHOICE, "AddressBookSort", &prefs.AddressBookSort, 0, (gpointer)0, addrbooksort_choices, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "KeepAddressesSorted", &prefs.KeepAddressesSorted, 0, (gpointer)TRUE, NULL, NULL, 0, },
	{ PREFS_TYPE_CHOICE, "SortGroupAs", &prefs.SortGroupsAs, 0, 0, sortorder_choices, NULL, 0, },
	{ PREFS_TYPE_BOOL, "AskBeforeQuitting", &prefs.AskBeforeQuitting, 0, (gpointer)TRUE, NULL, NULL, 0, },
	{ 0, },
};

struct PrefsTable msg_prefs_table[] =
{
	{ PREFS_TYPE_BOOL, "ExpungeDeleted", &prefs.ExpungeDeleted, 0, (gpointer)FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "OpenFirstNew", &prefs.OpenFirstNew, 0, (gpointer)FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "UseTagline", &prefs.UseTagline, 0, (gpointer)TRUE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "UseSignature", &prefs.UseSignature, 0, (gpointer)TRUE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "DisableReturnReceipt", &prefs.DisableReturnReceipt, 0, (gpointer)FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "CloseMsgOnReply", &prefs.CloseMsgOnReply, 0, (gpointer)FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "UseQuotedPrintable", &prefs.UseQuotedPrintable, 0, (gpointer)TRUE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "SignMsgs", &prefs.SignMsgs, 0, (gpointer)FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "EncryptMsgs", &prefs.EncryptMsgs, 0, (gpointer)FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "StripCodes", &prefs.StripCodes, 0, (gpointer)FALSE, NULL, NULL, NULL, },
	{ PREFS_TYPE_BOOL, "WrapText", &prefs.WrapText, 0, (gpointer)FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_INT, "DeleteSentMail", &prefs.DeleteSentMail, 0, (gpointer)FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "DisplayFilter", prefs.DisplayFilter, sizeof(prefs.DisplayFilter), "", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "HeaderOrder", prefs.HeaderOrder, sizeof(prefs.HeaderOrder), "From;To;Subject;Date", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "HideHeaders", prefs.HideHeaders, sizeof(prefs.HideHeaders), "Received", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "QuotePrefix", prefs.QuotePrefix, sizeof(prefs.QuotePrefix), "> ", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "ReplyHeader", prefs.ReplyHeader, sizeof(prefs.ReplyHeader), "", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "ForwardHeader", prefs.ForwardHeader, sizeof(prefs.ForwardHeader), "", NULL, NULL, 0, },
	{ PREFS_TYPE_INT, "MaxLineLength", &prefs.MaxLineLength, 0, (gpointer)72, NULL, NULL, 0, },
	{ PREFS_TYPE_CHOICE, "EOLType", &prefs.EOLType, 0, (gpointer)0, eoltype_choices, NULL, NULL, 0, },
	{ PREFS_TYPE_CHOICE, "ReplyDefault", &prefs.ReplyDefault, 0, (gpointer)0, replydefault_choices, NULL, NULL, 0, },
	{ PREFS_TYPE_CHOICE, "ForwardDefault", &prefs.ForwardDefault, 0, (gpointer)0, forwarddefault_choices, NULL, NULL, 0, },
	{ PREFS_TYPE_COMPOSITE, "PersonalPen", (gpointer)NULL, 0, "", NULL, parse_personal_pen, save_personal_pen, 0, },
	{ PREFS_TYPE_COMPOSITE, "PriorityPen", (gpointer)NULL, 0, "", NULL, parse_priority_pen, save_priority_pen, 0, },
	{ PREFS_TYPE_COMPOSITE, "ImportantPen", (gpointer)NULL, 0, "", NULL, parse_important_pen, save_important_pen, 0, },
	{ 0, },
};

struct PrefsTable user_prefs_table[] =
{
	{ PREFS_TYPE_STRING, "RealName", uprefs.RealName, sizeof(uprefs.RealName), "", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "Address", uprefs.Address, sizeof(uprefs.Address), "", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "ReplyTo", uprefs.ReplyTo, sizeof(uprefs.ReplyTo), "", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "Organization", uprefs.Organization, sizeof(uprefs.Organization), "", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "Signature", uprefs.Signature, sizeof(uprefs.Signature), ".signature", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "Taglines", uprefs.Taglines, sizeof(uprefs.Taglines), ".taglines", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "AddressBook", uprefs.AddressBook, sizeof(uprefs.AddressBook), ".addresses", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "MailDir", uprefs.MailDir, sizeof(uprefs.MailDir), "Mail", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "Mailcap", uprefs.Mailcap, sizeof(uprefs.Mailcap), ".mailcap", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "Filters", uprefs.Filters, sizeof(uprefs.Filters), ".filters", NULL, NULL, 0, },
	{ 0, },
};

struct PrefsTable servers_prefs_table[] =
{
	{ PREFS_TYPE_INT, "GetInterval", &prefs.GetInterval, 0, (gpointer)10, NULL, NULL, 0, },
	{ PREFS_TYPE_INT, "SendInterval", &prefs.SendInterval, 0, (gpointer)10, NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "SMTPServer", &prefs.SMTPServer, sizeof(prefs.SMTPServer), "", NULL, NULL, 0, },
	{ PREFS_TYPE_INT, "SMTPPort", &prefs.SMTPPort, 0, (gpointer)25, NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "SMTPCommand", prefs.SMTPCommand, sizeof(prefs.SMTPCommand), "", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "NewMailAction", prefs.NewMailAction, sizeof(prefs.NewMailAction), "", NULL, NULL, 0, },
	{ PREFS_TYPE_STRING, "NoMailAction", prefs.NoMailAction, sizeof(prefs.NoMailAction), "", NULL, NULL, 0, },
	{ PREFS_TYPE_CHOICE, "SMTPType", &prefs.SMTPType, 0, (gpointer)0, smtptype_choices, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "InitialMailSend", &prefs.InitialMailSend, 0, FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_BOOL, "InitialMailGet", &prefs.InitialMailGet, 0, FALSE, NULL, NULL, 0, },
	{ PREFS_TYPE_CHOICE, "POPWindowType", &prefs.POPWindowType, 0, (gpointer)0, windowtype_choices, NULL, NULL, 0, },
	{ PREFS_TYPE_ITER, "Retriever", (gpointer)0, 0, "", NULL, parse_retriever, save_retriever, PTF_SKIPDEFAULT, },
	{ 0, },
};

struct PrefsTable windows_prefs_table[] =
{
	{ PREFS_TYPE_COMPOSITE, "MainWindowPos", (gpointer)NULL, 0, "", NULL, parse_main_window_pos, save_main_window_pos, 0, },
	{ PREFS_TYPE_COMPOSITE, "MsgWindowPos", (gpointer)NULL, 0, "", NULL, parse_msg_window_pos, save_msg_window_pos, 0, },
	{ PREFS_TYPE_COMPOSITE, "ComposeWindowPos", (gpointer)NULL, 0, "", NULL, parse_compose_window_pos, save_compose_window_pos, 0, },
	{ PREFS_TYPE_COMPOSITE, "FolderWindowPos", (gpointer)NULL, 0, "", NULL, parse_folder_window_pos, save_folder_window_pos, 0, },
	{ PREFS_TYPE_COMPOSITE, "POPWindowPos", (gpointer)NULL, 0, "", NULL, parse_pop_window_pos, save_pop_window_pos, 0, },
	{ PREFS_TYPE_COMPOSITE, "SMTPWindowPos", (gpointer)NULL, 0, "", NULL, parse_smtp_window_pos, save_smtp_window_pos, 0, },
	{ 0, },
};

struct PrefsFile pfiles[] =
{
	{ "general.prefs", general_prefs_table, /* prefs_general_set_defaults,*/  },
	{ "user.prefs", user_prefs_table, /* prefs_user_set_defaults,*/  },
	{ "msg.prefs", msg_prefs_table, /* prefs_mail_set_defaults,*/  },
	{ "server.prefs", servers_prefs_table, /* prefs_network_set_defaults, */ },
	{ "window.prefs", windows_prefs_table, /* prefs_windows_set_defaults,*/  },
	{ 0, },
};

enum
{
	PREFS_GENERAL,
	PREFS_USER,
	PREFS_MSG,
	PREFS_SERVER,
	PREFS_WINDOW,
	PREFS_MAX
};

gboolean prefs_init(void)
{
	glong i;
	
	memset(&prefs, 0, sizeof(struct Prefs));
	memset(&uprefs, 0, sizeof(struct User));
	
	/* Set defaults.
	 */
	for(i=0; pfiles[i].Filename; i++)
	{
		struct PrefsTable *ptable = pfiles[i].Table;
		glong j;
		
		/* Do some basic initialization.
		 */
		if(pfiles[i].InitFunc)
			pfiles[i].InitFunc ();
		
		/* Set up the defaults.
		 */
		for(j=0; ptable[j].Type; j++)
		{
			if(ptable[j].Flags & PTF_SKIPDEFAULT)
				continue;

			switch(ptable[j].Type)
			{
				case PREFS_TYPE_STRING:
					strncpy((gchar *)ptable[j].Storage, ptable[j].Default ? ptable[j].Default : "", ptable[j].Size - 1);
					break;
					
				case PREFS_TYPE_BOOL:
					*(gboolean *)ptable[j].Storage = (gboolean)GPOINTER_TO_INT(ptable[j].Default);
					break;
					
				case PREFS_TYPE_INT:
					*(glong *)ptable[j].Storage = (glong)GPOINTER_TO_INT(ptable[j].Default);
					break;
					
				case PREFS_TYPE_CHOICE:
					*(gchar *)ptable[j].Storage = (gchar)GPOINTER_TO_INT(ptable[j].Default);
					break;
					
				case PREFS_TYPE_COMPOSITE:
				case PREFS_TYPE_ITER:
					ptable[j].ParseFunc (ptable[j].Default);
					break;
			}
		}
	}
	
	return TRUE;
}

gboolean prefs_load(void)
{
	glong i;
	gchar *euc_path, *euc_user_path, *config_file = NULL;
	gboolean success = TRUE;
	FILE *fp;

	euc_path = path_build(g_get_home_dir(), ".eucalyptus", NULL);
	euc_user_path = path_build(euc_path, "user.prefs", NULL);
	
	if(!path_is_dir(euc_path) ||
	   !path_is_file(euc_user_path))
	{
		/* It doesn't appear that they've used Eucalyptus
		 * before (poor soul), but there is hope!
		 * Create a directory and ...
		 */
		if(path_create_directory(euc_path))
		{
			/* ... go through initial configuration, using
			 * that nifty GnomeDruid widget thingy.  Yeah!
			 * FIXME
			 */
			if(prefs_new_config())
			{
				g_free(euc_path);
				g_free(euc_user_path);
				return FALSE;
			}
		}
		else
		{
			const gchar *buttons[] = { N_("Doh!"), NULL, };
			
			euc_request(NULL, 
					_("Eucalyptus error"), 
					_("Can't create ~/.eucalyptus directory!"), 
					buttons, 0,
					0);
		}
	}
	g_free(euc_user_path);

	for(i=0; pfiles[i].Filename; i++)
	{
		struct PrefsTable *ptable = pfiles[i].Table;
		glong j, k;

		if(config_file)
			g_free(config_file);
		config_file = path_build(euc_path, pfiles[i].Filename, NULL);

		if(fp = fopen(config_file, "r"))
		{
			gchar buf[1024];
			
			while(fgets(buf, sizeof(buf), fp))
			{
				gchar *c, *item, *param;
				
				/* Check for a comment.
				 */
				if(*buf == '#')
					continue;
				
				/* Strip ends-of-line.
				 */
				if(c = strchr(buf, '\r'))
					*c = 0;
				if(c = strchr(buf, '\n'))
					*c = 0;
					
				item = strip_whitespace(buf);
					
				/* Check if there's nothing here.
				 */
				if(!(*item))
					continue;
					
				/* Parse out the data.
				 */
				if(c = strchr(item, '='))
				{
					*c = 0;
					
					strip_trailing_blanks(item);
					
					param = strip_leading_blanks(c + 1);
				}
				else
					param = "true";
					
				if(!(*param))
					continue;
					
				for(j=0; ptable[j].Type; j++)
				{
					if(!strcasecmp(ptable[j].Item, item))
					{
						switch(ptable[j].Type)
						{
							gchar *gstr;
							
							case PREFS_TYPE_STRING:
								strncpy((gchar *)ptable[j].Storage, param, ptable[j].Size - 1);
								break;
								
							case PREFS_TYPE_BOOL:
								*(gboolean *)ptable[j].Storage = (gboolean)(!strcasecmp(param, "true"));
								break;
								
							case PREFS_TYPE_INT:
								*(glong *)ptable[j].Storage = (gboolean)atoi(param);
								break;
								
							case PREFS_TYPE_COMPOSITE:
							case PREFS_TYPE_ITER:
								ptable[j].ParseFunc (param);
								break;
								
							case PREFS_TYPE_CHOICE:
								for(k=0; ptable[j].Choices[k]; k++)
								{
									if(!strcasecmp(ptable[j].Choices[k], param))
									{
										*(gchar *)ptable[j].Storage = (gchar)k;
										break;
									}
								}
								break;
						}
					}
				}
			}
			
			fclose(fp);
		}
		else
			success = FALSE;
	}

	g_free(config_file);
	g_free(euc_path);

	return success;
}

gboolean prefs_save(void)
{
	glong i;
	gboolean success = TRUE;
	
	for(i=0; pfiles[i].Filename; i++)
	{
		if(!prefs_save_section(i))
		{
			success = FALSE;
			break;
		}
	}

	return success;
}

gboolean prefs_save_section(gchar i)
{
	struct PrefsTable *ptable = pfiles[i].Table;
	glong j;
	gchar *config_file = NULL, *euc_path;
	gboolean success = TRUE;
	FILE *fp;

	euc_path = path_build(g_get_home_dir(), ".eucalyptus", NULL);
	config_file = path_build(euc_path, pfiles[i].Filename, NULL);

	if(fp = fopen(config_file, "w"))
	{
		for(j=0; ptable[j].Type; j++)
		{
			fprintf(fp, "%s = ", ptable[j].Item);
			
			switch(ptable[j].Type)
			{
				gchar *gstr;
				glong k;
				
				case PREFS_TYPE_STRING:
					fputs((gchar *)ptable[j].Storage, fp);
					break;
					
				case PREFS_TYPE_BOOL:
					fputs(*(gboolean *)ptable[j].Storage ? "true" : "false", fp);
					break;
					
				case PREFS_TYPE_INT:
					fprintf(fp, "%ld", *(glong *)ptable[j].Storage);
					break;
					
				case PREFS_TYPE_COMPOSITE:
					if(gstr = ptable[j].SaveFunc ())
					{
						fputs(gstr, fp);
					
						g_free(gstr);
					}
					else
						fputs(ptable[j].Default, fp);
					break;
					
				case PREFS_TYPE_CHOICE:
					fputs(ptable[j].Choices[*(gchar *)ptable[j].Storage], fp);
					break;
					
				case PREFS_TYPE_ITER:
					for(k=0; ; k++)
					{
						if(gstr = ptable[j].SaveFunc (k))
						{
							fputs(gstr, fp);
						
							g_free(gstr);
						}
						else
							break;
					}
					break;
			}
			
			fputs("\n", fp);
		}

		fclose(fp);
	}
	else
		success = FALSE;

	g_free(config_file);
	g_free(euc_path);

	return success;
}

/* Parsing functions.
 */
gboolean parse_main_window_pos(gchar *data)
{
	sscanf(data, "%d/%d/%d/%d", 
				    &prefs.MainWindowLeft,
				    &prefs.MainWindowTop,
				    &prefs.MainWindowWidth,
				    &prefs.MainWindowHeight);

	return TRUE;
}

gchar *save_main_window_pos(void)
{
	return g_strdup_printf("%d/%d/%d/%d",
				 prefs.MainWindowLeft,
				 prefs.MainWindowTop,
				 prefs.MainWindowWidth,
				 prefs.MainWindowHeight);
}

gboolean parse_msg_window_pos(gchar *data)
{
	sscanf(data, "%d/%d/%d/%d", 
				    &prefs.MsgWindowLeft,
				    &prefs.MsgWindowTop,
				    &prefs.MsgWindowWidth,
				    &prefs.MsgWindowHeight);

	return TRUE;
}

gchar *save_msg_window_pos(void)
{
	return g_strdup_printf("%d/%d/%d/%d",
				 prefs.MsgWindowLeft,
				 prefs.MsgWindowTop,
				 prefs.MsgWindowWidth,
				 prefs.MsgWindowHeight);
}

gboolean parse_compose_window_pos(gchar *data)
{
	sscanf(data, "%d/%d/%d/%d", 
				    &prefs.ComposeWindowLeft,
				    &prefs.ComposeWindowTop,
				    &prefs.ComposeWindowWidth,
				    &prefs.ComposeWindowHeight);

	return TRUE;
}

gchar *save_compose_window_pos(void)
{
	return g_strdup_printf("%d/%d/%d/%d",
				 prefs.ComposeWindowLeft,
				 prefs.ComposeWindowTop,
				 prefs.ComposeWindowWidth,
				 prefs.ComposeWindowHeight);
}

gboolean parse_folder_window_pos(gchar *data)
{
	sscanf(data, "%d/%d/%d/%d", 
				    &prefs.FolderWindowLeft,
				    &prefs.FolderWindowTop,
				    &prefs.FolderWindowWidth,
				    &prefs.FolderWindowHeight);

	return TRUE;
}

gchar *save_folder_window_pos(void)
{
	return g_strdup_printf("%d/%d/%d/%d",
				 prefs.FolderWindowLeft,
				 prefs.FolderWindowTop,
				 prefs.FolderWindowWidth,
				 prefs.FolderWindowHeight);
}

gboolean parse_pop_window_pos(gchar *data)
{
	sscanf(data, "%d/%d/%d/%d", 
				    &prefs.POPWindowLeft,
				    &prefs.POPWindowTop,
				    &prefs.POPWindowWidth,
				    &prefs.POPWindowHeight);

	return TRUE;
}

gchar *save_pop_window_pos(void)
{
	return g_strdup_printf("%d/%d/%d/%d",
				 prefs.POPWindowLeft,
				 prefs.POPWindowTop,
				 prefs.POPWindowWidth,
				 prefs.POPWindowHeight);
}

gboolean parse_smtp_window_pos(gchar *data)
{
	sscanf(data, "%d/%d/%d/%d", 
				    &prefs.SMTPWindowLeft,
				    &prefs.SMTPWindowTop,
				    &prefs.SMTPWindowWidth,
				    &prefs.SMTPWindowHeight);

	return TRUE;
}

gchar *save_smtp_window_pos(void)
{
	return g_strdup_printf("%d/%d/%d/%d",
				 prefs.SMTPWindowLeft,
				 prefs.SMTPWindowTop,
				 prefs.SMTPWindowWidth,
				 prefs.SMTPWindowHeight);
}

gboolean parse_personal_pen(gchar *data)
{
	sscanf(data, "%d/%d/%d",
				    &prefs.PersonalPen.red,
				    &prefs.PersonalPen.green,
				    &prefs.PersonalPen.blue);
}

gchar *save_personal_pen(void)
{
	return g_strdup_printf("%d/%d/%d",
				    prefs.PersonalPen.red,
				    prefs.PersonalPen.green,
				    prefs.PersonalPen.blue);
}

gboolean parse_priority_pen(gchar *data)
{
	sscanf(data, "%d/%d/%d",
				    &prefs.PriorityPen.red,
				    &prefs.PriorityPen.green,
				    &prefs.PriorityPen.blue);
}

gchar *save_priority_pen(void)
{
	return g_strdup_printf("%d/%d/%d",
				    prefs.PriorityPen.red,
				    prefs.PriorityPen.green,
				    prefs.PriorityPen.blue);
}

gboolean parse_important_pen(gchar *data)
{
	sscanf(data, "%d/%d/%d",
				    &prefs.ImportantPen.red,
				    &prefs.ImportantPen.green,
				    &prefs.ImportantPen.blue);
}

gchar *save_important_pen(void)
{
	return g_strdup_printf("%d/%d/%d",
				    prefs.ImportantPen.red,
				    prefs.ImportantPen.green,
				    prefs.ImportantPen.blue);
}

gboolean parse_retriever(gchar *data)
{
	Server *server;
	gchar *c, *v;

	server = pop_server_new(0);

	if(!server)
		return FALSE;

	  v = data;
	  if(c = strchr(v, ':'))
	  {
		  *c = '\0';

		  if(!strcasecmp("INTERNAL", v))
		  {
			  server->Type = PST_INTERNAL;

			  v = c + 1;

			  if(c = strchr(v, ':'))
			  {
				  *c = 0;
				  strncpy(server->Host, strip_leading_blanks(v), sizeof(server->Host) - 1);
				  v = c + 1;

				  if(c = strchr(v, ':'))
				  {
					  *c = '\0';
					  server->Port = atol(v);
					  v = c + 1;

					  if(c = strchr(v, ':'))
					  {
						  *c =  0;

						  if(!strcasecmp(v, "POP3"))
							  server->Type = PST_POP3;
						  else if(!strcasecmp(v, "IMAP4"))
							  server->Type = PST_IMAP4;

						  v = c + 1;

						  if(c = strchr(v, ':'))
						  {
								glong i, l;

							  *c = '\0';
							  strncpy(server->Username, v, sizeof(server->Username) - 1);
							  v = c + 1;

							  if(c = strchr(v, ':'))
								  *c = 0;

						    l=strlen(v)/2;

						    for(i=0; i<l; i++)
						    {
							    glong vv;
							    gchar hv[3], ch[3];

							    hv[0] = v[i * 2];
							    hv[1] = v[i * 2 + 1];
							    hv[2] = '\0';
							    vv = strtol(hv, NULL, 16);
							    {
								    g_snprintf(ch, sizeof(ch), "%c", (gchar)vv);
								    strncat(server->Password, ch, sizeof(server->Password) - 1);
							    }
						    }

							  if(c)
							  {
								  gboolean endofs = FALSE;

								  v = c + 1;
								  if(c = strchr(v, ':'))
								  {
									  *c = 0;
									  if(!strcasecmp(v, "POP3"))
										  server->Protocol = PST_POP3;
									  else if(!strcasecmp(v, "IMAP4"))
										  server->Protocol = PST_IMAP4;

									  v = c + 1;

									  while(!endofs)
									  {
										  if(c = strchr(v, ':'))
											  *c = 0;
										  else
											  endofs = TRUE;

										  if(!strstr(v, "NOFILTERS"))
											  server->IgnoreFilters = TRUE;
										  if(!strstr(v, "DISABLED"))
											  server->Disabled = TRUE;
										  if(!strstr(v, "NOHEADERS"))
											  server->DontRetrieveHeaders = TRUE;
										  if(!strstr(v, "AUTO"))
											  server->AutoRetrieve = TRUE;
										  if(!strstr(v, "DELETE"))
											  server->DeleteOnServer = TRUE;
										  if(!strncasecmp(v, "LIMIT=", 6))
										  {
											  gchar *sla;

											  server->SizeLimit = atol(v + 6);

											  if(sla = strchr(v + 6, '/'))
											  {
												  sla++;

												  if(!strcasecmp(sla, "ASK"))
													  server->SizeLimitAction = SLAT_ASK;
												  else if(!strcasecmp(sla, "SKIP"))
													  server->SizeLimitAction = SLAT_SKIP;
												  else if(!strcasecmp(sla, "DELETE"))
													  server->SizeLimitAction = SLAT_DELETE;
											  }
										  }

										  v = c + 1;
									  }
								  }
							  }
						  }
					  }
				  }
			  }
		  }
		  else if(!strcasecmp("COMMAND", v))
		  {
			  server->Type = PST_COMMAND;

			  strncpy(server->Command, strip_leading_blanks(c + 1), sizeof(server->Command) - 1);
		  }
	}

	list_append(&euc.Servers, server);

	return TRUE;
}

gchar *save_retriever(glong num)
{
	Server *server;
	glong i;
	gchar *gstr = NULL, *g2, pw[101], limit[51];
	gchar *server_types[] = { "INTERNAL", "COMMAND", };
	gchar *server_protocols[] = { "POP3", "IMAP4", };
	gchar *limit_actions[] = { "ASK", "SKIP", "DELETE", };

	server = g_list_nth_data(euc.Servers.List, num);

	if(!server)
		   return NULL;

	memset(pw, 0, sizeof(pw));
	for(i=0; server->Password[i]; i++)
	{
		gchar buf[5];

		g_snprintf(buf, sizeof(buf), "%.2X", server->Password[i]);
		strncat(pw, buf, sizeof(pw) - 1);
	}

	g_snprintf(limit, sizeof(limit), ":LIMIT=%d/%s", 
				 server->SizeLimit, 
				 limit_actions[server->SizeLimitAction]);

	gstr = g_strdup_printf("%s:%s:%d:%s:%s:%s%s%s%s%s%s%s",
				 server_types[server->Type],
				 server->Host,
				 server->Port,
				 server_protocols[server->Protocol],
				 server->Username,
				 pw,
				 server->IgnoreFilters ? ":NOFILTERS" : "",
				 server->Disabled ? ":DISABLED" : "",
				 server->AutoRetrieve ? ":AUTO" : "",
				 server->DontRetrieveHeaders ? ":NOHEADERS" : "",
				 server->DeleteOnServer ? ":DELETE" : "",
				 limit);

	return gstr;
}


