/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: smtp.c
 *   	 SMTP client function server
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <netinet/in.h>
#include <netdb.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "eucalyptus.h"

struct SMTPControl smtp_control;

void *smtp_thread(void *arg)
{
	ProgressMsg *pmsg;
	eMsg *emsg = NULL;
	GList *node;
	glong i, type, progress = 0;

	type = prefs.SMTPType;

	/* Message processing loop.
	 */
	for(;;)
	{
		SendMsg *smsg;
		Folder *folder;
		GList *node;
		struct stat *st = NULL;
		int rc;
		gchar *dir, *fname;
		gboolean msg_sent = FALSE, nomsgs = FALSE;

		if(emsg)
			g_free(emsg);

		if(smtp_control.Flags & SCF_STOP)
			   break;

		/* Grab a waiting message.
		 */
		pthread_mutex_lock(&smtp_control.th.Mutex);
		if(g_list_length(smtp_control.Outgoing.List))
		{
			/* Grab the next item.
			 */
			node = g_list_first(smtp_control.Outgoing.List);
			emsg = (eMsg *)node->data;
			
			/* Remove it from the list.
			 */
			smtp_control.Outgoing.List = list_remove(&smtp_control.Outgoing, emsg);
		}
		else
			nomsgs = TRUE;
		pthread_mutex_unlock(&smtp_control.th.Mutex);
		
		if(nomsgs)
			break;
		
		/* Check flags and action.
		 */
		if(emsg->type == MSG_TYPE_DIE)
		{
			g_free(emsg);
			break;
		}
		
		if(emsg->type != MSG_TYPE_SEND)
			continue;

		smsg = (SendMsg *)emsg;

		if(smsg->flags & CMF_SKIP)
			continue;

		/* Get the full path.
		 */
		folder = folder_get_special(FTF_OUTGOING);
		dir = folder_get_full_path(folder);
		fname = path_build(dir, smsg->filename, NULL);

		/* Send a visual update.
		 */
		if(pmsg = g_malloc0(sizeof(ProgressMsg)))
		{
			pmsg->msg.type = MSG_TYPE_PROGRESS;
			pmsg->msg.subtype = MSG_SUBTYPE_PROGRESS_OPEN;
			pmsg->who = MSG_WHO_SMTP;
			pmsg->total = smtp_control.Size;
			pmsg->current = smtp_control.Level;
			strncpy(pmsg->id, smsg->id, sizeof(pmsg->id) - 1);
			/* text */

			pthread_mutex_lock(&euc.q.Mutex);
			list_append(&euc.q.Msgs, pmsg);
			pthread_mutex_unlock(&euc.q.Mutex);
		}

		/* Determine what to do with it.
		 */
		switch(type)
		{
			case PST_INTERNAL:
				msg_sent = smtp_send_msg_internal(fname);
				break;
				
			case PST_COMMAND:
				msg_sent = smtp_send_msg_command(fname);
				break;
		}
		
		/* Check if it was sent successfully.
		 */
		if(msg_sent)
		{
			CopyMsg *response;
			
			/* Formulate a response to the main
			 * thread, so it moves the sent message
			 * to the "Sent" folder.
			 */
			if(response = g_malloc0(sizeof(CopyMsg)))
			{
				response->msg.type = MSG_TYPE_COPY;
				response->move = TRUE;
				response->source = folder_get_special(FTF_OUTGOING);
				response->dest = folder_get_special(FTF_SENT);
				strncpy(response->filename, fname, sizeof(response->filename) - 1);

				pthread_mutex_lock(&euc.q.Mutex);
				list_append(&euc.q.Msgs, response);
				pthread_mutex_unlock(&euc.q.Mutex);
			}
		}
		
		/* Clean up.
		 */
		g_free(fname);
		g_free(dir);
	}

	smtp_exit:
	/* Do some cleanup.
	 */
	if(type == PST_INTERNAL)
		smtp_close_server();

	smtp_control.Flags &= ~(SCF_STOP);
	smtp_control.th.Thread = 0;
	
	/* Free the msg queue.
	 */
	while(node = g_list_first(smtp_control.Outgoing.List))
	{
		eMsg *emsg;
		
		emsg = (eMsg *)node->data;
		smtp_control.Outgoing.List = list_remove(&smtp_control.Outgoing, emsg);
		g_free(emsg);
	}

	/* Send a close message.
	 */
	if(pmsg = g_malloc0(sizeof(ProgressMsg)))
	{
		pmsg->msg.type = MSG_TYPE_PROGRESS;
		pmsg->msg.subtype = MSG_SUBTYPE_PROGRESS_CLOSE;
		pmsg->who = MSG_WHO_SMTP;

		pthread_mutex_lock(&euc.q.Mutex);
		list_append(&euc.q.Msgs, pmsg);
		pthread_mutex_unlock(&euc.q.Mutex);
	}

	return NULL;
}

gboolean smtp_send_msg_internal(gchar *fname)
{
	struct List *hdrs = NULL;
	glong val;
	gchar smtp_domain[51], in_buf[1024], out_buf[1024];
	gboolean success = TRUE;

	/* Check for a connection to the server.
	 * If not, initiate one.
	 */
	if(!smtp_control.Connected)
	{
		struct hostent *host;

		memset(smtp_domain, 0, sizeof(smtp_domain));
		getdomainname(smtp_domain, sizeof(smtp_domain));
// FIXME	GetVar("SMTPHOST", smtp_domain, sizeof(smtp_domain) - 1, 0);
		if(!*smtp_domain)
			strncpy(smtp_domain, prefs.SMTPServer, sizeof(smtp_domain) - 1);

		memset(&smtp_control.Server, 0, sizeof(smtp_control.Server));

		smtp_set_status(NULL, _("Connecting to mail server..."), 0);

		if(!(*prefs.SMTPServer))
		{
			if(host = gethostbyname("SMTPHOST"))
			{
				strncpy(prefs.SMTPServer, host->h_name, sizeof(prefs.SMTPServer) - 1);
			}
		}

		smtp_control.Server.sin_family = AF_INET;
		smtp_control.Server.sin_port = htons(prefs.SMTPPort);
		smtp_control.Server.sin_addr.s_addr = 0;

		if(host = gethostbyname(prefs.SMTPServer))
		{
			glong signals;
			
			memcpy(&smtp_control.Server.sin_addr, host->h_addr, sizeof(smtp_control.Server.sin_addr));

			if(-1 != (smtp_control.Sock = socket(smtp_control.Server.sin_family, SOCK_STREAM, 0)))
			{
				if(0 == connect(smtp_control.Sock, (struct sockaddr *)&smtp_control.Server, sizeof(smtp_control.Server)))
				{
					smtp_control.Connected = TRUE;

					/* Wait for greeting.
					 */
					if(220 == smtp_send(smtp_control.Sock, "", in_buf, sizeof(in_buf)))
					{
						/* FIXME
						 * We need to check the return greeting
						 * from the server, to see if it can handle
						 * ESMTP, and act accordingly.
						 */
						smtp_set_status(NULL, _("Sending greeting..."), 0);

						g_snprintf(out_buf, sizeof(out_buf), "HELO %s\r\n", smtp_domain);
						switch(val = smtp_send(smtp_control.Sock, out_buf, in_buf, sizeof(in_buf)))
						{
							case 250: 	// good
								smtp_set_status(NULL, _("Connection established."), 0);
								break;
							default:
								shutdown(smtp_control.Sock, 2);
								smtp_control.Sock = -1;
								smtp_control.Connected = FALSE;
								smtp_set_status(NULL, _("Unable to connect!"), 0);
								break;
						}
					}
					else
						success = FALSE;
				}
				else
				{
					success = FALSE;
				}
			}
			else
			{
				success = FALSE;
			}
		}
		else
		{
			success = FALSE;
		}
	}

	/* Send the message.
	 */
	if(success)
	{
		gchar *afile = NULL, *c;
		FILE *fp;
		
		/* Check for an address file.  If present,
		 * get the addresses from it.  Otherwise,
		 * get the addresses straight from the
		 * message.
		 */
		afile = g_strdup(fname);
		c = strstr(afile, G_DIR_SEPARATOR_S "msg");
		memcpy(c + 1, "__a", 3);
		
		if(fp = fopen(afile, "r"))
		{
			fp = fopen(fname, "r");
		}

		if(fp)
		{
			hdrs = mailmsg_get_header(GMHT_FILE, fp, NULL);
			
			/* Send the message.
			 */
			if(hdrs)
			{
				success = smtp_send_mail_file(smtp_control.Sock, fname, hdrs);

				mailmsg_free_header(hdrs);
			}

			fclose(fp);
		}
		
		g_free(afile);
	}
	
	return success;
}

gboolean smtp_send_msg_command(gchar *fname)
{
	glong x = 0;
	gchar cmd[2048];
	gboolean success = TRUE;

	while(prefs.SMTPCommand[x])
	{
		switch(prefs.SMTPCommand[x])
		{
			gchar work[21];

			case '%':
				switch(prefs.SMTPCommand[++x])
				{
					case '%':
						strncat(cmd, "%", sizeof(cmd) - 1);
						break;
					case 'f':
						strncat(cmd, fname, sizeof(cmd) - 1);
						break;
					case 'h':
						strncat(cmd, prefs.SMTPServer, sizeof(cmd) - 1);
						break;
					case 'p':
						g_snprintf(work, sizeof(work), "%ld", prefs.SMTPPort);
						strncat(cmd, work, sizeof(cmd) - 1);
						break;
				}
				break;
			default:
				g_snprintf(work, sizeof(work), "%c", prefs.SMTPCommand[x]);
				strncat(cmd, work, sizeof(cmd) - 1);
				break;
		}

		x++;
	}

	switch(x = gnome_execute_shell(NULL, cmd))
	{
		case 0:
			break;
		default:
//			smtp_set_item_status(node, LocaleString(ETXT_SMTP_QUEUESTATUS_UNABLETOSEND), 0);
			success = FALSE;
			break;
	}
	
	return success;
}

gboolean smtp_send_mail_file(int sock, gchar *fname, struct List *hdrs)
{
	HdrNode *hnode, *from;
	struct List *to = NULL, *cc = NULL, *bcc = NULL;
	glong error = 0;
	gchar in_buf[1024], out_buf[1024];
	gboolean success = TRUE;
	FILE *fp = NULL;

	if((sock < 0) ||
	   !fname ||
	   !hdrs)
		return FALSE;

	if(from = header_find_tag("From", hdrs, 0))
	{
		gchar fromaddr[101];

		parse_address(from->Data, NULL, 0, fromaddr, sizeof(fromaddr), FALSE);

		if(hnode = header_find_tag("To", hdrs, 0))
		{
			if(to = smtp_parse_addressees(hnode->Data, 0))
			{
				GList *node;
				glong val, signals;
				gboolean okaytosend = FALSE;

				if(hnode = header_find_tag("Cc", hdrs, 0))
					cc = smtp_parse_addressees(hnode->Data, 0);
				if(hnode = header_find_tag("Bcc", hdrs, 0))
					bcc = smtp_parse_addressees(hnode->Data, 0);

				do_mail:
				smtp_set_status(NULL, _("Negotiating..."), 0);

				g_snprintf(out_buf, sizeof(out_buf), "MAIL FROM:<%s>\r\n", fromaddr);

				switch(val = smtp_send(sock, out_buf, in_buf, sizeof(in_buf)))
				{
					case 250: 	// good
						break;
					default:
						smtp_set_status(NULL, _("Negotiation failed!"), 0);
						success = FALSE;
						send(sock, "RSET\r\n", 6, 0);
						goto smtp_out;
				}

				// to addresses
				if(to)
				{
					node = g_list_first(to->List);
					for(; node; node = g_list_next(node))
					{
						gchar *str = (gchar *)node->data;
						
						if(str)
						{
							gchar sendto[201], tmpbuf[101], *name, try = 0;

							name = str;

							try_again:
							if(name && *name)
							{
								g_snprintf(sendto, sizeof(sendto), _("Sending to '%s'..."), name);
								smtp_set_status(NULL, sendto, 0);

								g_snprintf(out_buf, sizeof(out_buf), "RCPT TO:<%s>\r\n", name);
								switch(val = smtp_send(sock, out_buf, in_buf, sizeof(in_buf)))
								{
									gchar *c, *e;

									case 250: 	// good
									case 251:
										okaytosend = TRUE;
										break;
									case 551: 	// try different address
										if(try == 0)
										{
											if(c = strchr(in_buf, '<'))
											{
												if(e = strchr(c, '>'))
												{
													*e = '\0';
													strncpy(tmpbuf, c + 1, sizeof(tmpbuf) - 1);
													name = tmpbuf;
													try = 1;
													goto try_again;
												}
											}
										}
									default:
										smtp_set_status(NULL, in_buf, 0);
										success = FALSE;
										send(sock, "RSET\r\n", 6, 0);
										goto smtp_out;
								}
							}
						}
#if 0
						if(control->Flags & (SCF_SUSPEND | SCF_STOP))
							goto smtp_out;
#endif
					}
				}
				// cc addresses
				if(cc)
				{
					node = g_list_first(cc->List);
					for(; node; node = g_list_next(node))
					{
						gchar *str = (gchar *)node->data;
						
						if(str)
						{
							gchar sendto[201], tmpbuf[101], *name, try = 0;

							name = str;

							try_again_cc:
							if(name && *name)
							{
								g_snprintf(sendto, sizeof(sendto), _("Sending carbon copy to '%s'..."), name);
								smtp_set_status(NULL, sendto, 0);

								g_snprintf(out_buf, sizeof(out_buf), "RCPT TO:<%s>\r\n", name);
								switch(val = smtp_send(sock, out_buf, in_buf, sizeof(in_buf)))
								{
									gchar *c, *e;

									case 250: 	// good
									case 251:
										okaytosend = TRUE;
										break;
									case 551: 	// try different address
										if(try == 0)
										{
											if(c = strchr(in_buf, '<'))
											{
												if(e = strchr(c, '>'))
												{
													*e = '\0';
													strncpy(tmpbuf, c + 1, sizeof(tmpbuf) - 1);
													name = tmpbuf;
													try = 1;
													goto try_again_cc;
												}
											}
										}
									default:
										smtp_set_status(NULL, in_buf, 0);
										success = FALSE;
										send(sock, "RSET\r\n", 6, 0);
										goto smtp_out;
								}
							}
						}
#if 0
						if(control->Flags & (SCF_SUSPEND | SCF_STOP))
							goto smtp_out;
#endif
					}
				}
				// bcc addresses
				if(bcc)
				{
					node = g_list_first(bcc->List);
					for(; node; node = g_list_next(node))
					{
						gchar *str = (gchar *)node->data;
						
						if(str)
						{
							gchar sendto[201], tmpbuf[101], *name, try = 0;

							name = str;

							try_again_bcc:
							if(name && *name)
							{
								g_snprintf(sendto, sizeof(sendto), _("Sending blind carbon copy to '%s'..."), name);
								smtp_set_status(NULL, sendto, 0);

								g_snprintf(out_buf, sizeof(out_buf), "RCPT TO:<%s>\r\n", name);
								switch(val = smtp_send(sock, out_buf, in_buf, sizeof(in_buf)))
								{
									gchar *c, *e;

									case 250: 	// good
									case 251:
										okaytosend = TRUE;
										break;
									case 551: 	// try different address
										if(try == 0)
										{
											if(c = strchr(in_buf, '<'))
											{
												if(e = strchr(c, '>'))
												{
													*e = '\0';
													strncpy(tmpbuf, c + 1, sizeof(tmpbuf) - 1);
													name = tmpbuf;
													try = 1;
													goto try_again_bcc;
												}
											}
										}
									default:
										smtp_set_status(NULL, in_buf, 0);
										success = FALSE;
										send(sock, "RSET\r\n", 6, 0);
										goto smtp_out;
								}
							}
						}
#if 0
						if(control->Flags & (SCF_SUSPEND | SCF_STOP))
							goto smtp_out;
#endif
					}
				}

				if(okaytosend)
				{
					glong size = 0;

					switch(val = smtp_send(sock, "DATA\r\n", in_buf, sizeof(in_buf)))
					{
						struct stat *st;

						case 354: 	// good
							smtp_set_status(NULL, _("Sending message..."), 0);

							if(st = path_stat(fname))
							{
								smtp_control.Size = st->st_size;
								g_free(st);
							}
							
							size = 0;
							if(fp = fopen(fname, "r"))
							{
								gboolean bccing = FALSE;

								while(fgets(out_buf, sizeof(out_buf) - 1, fp))
								{
									glong signals, oblen;

									smtp_set_status(NULL, NULL, size += strlen(out_buf));

									if(*out_buf == '.')
										insert_string(out_buf, ".", sizeof(out_buf));

									if(!strncasecmp(out_buf, "Bcc:", 4))
									{
										bccing = TRUE;
										continue;
									}

									if(((*out_buf == ' ') ||
									    (*out_buf == '\t')) &&
									    bccing)
									{
										continue;
									}

									bccing = FALSE;

									oblen = strlen(out_buf);
									if((oblen < 2) ||
									   ((out_buf[oblen - 2] != '\r') &&
									    (out_buf[oblen - 1] != '\n')))
									{
										if(out_buf[oblen - 1] == '\r')
											strncat(out_buf, "\n", sizeof(out_buf) - 1);
										else if(out_buf[oblen - 1] == '\n')
										{
											out_buf[oblen - 1] = '\0';
											strncat(out_buf, "\r\n", sizeof(out_buf) - 1);
										}
										else
											strncat(out_buf, "\r\n", sizeof(out_buf) - 1);
									}
									if(-1 == send(sock, out_buf, oblen, 0))
									{
										success = FALSE;
										goto send_out;
									}
								}

								switch(val = smtp_send(sock, "\r\n.\r\n", in_buf, sizeof(in_buf)))
								{
									case 250: 	// good
										break;
									default:
										smtp_set_status(NULL, in_buf, 0);
										success = FALSE;
										send(sock, "RSET\r\n", 6, 0);
										break;
								}

								send_out:
								fclose(fp);
								fp = NULL;
							}
							break;
						default:
							smtp_set_status(NULL, in_buf, 0);
							success = FALSE;
							goto smtp_out;
					}
				}
			}
			else
			{
				success = FALSE;
			}

			smtp_out:
			if(fp)
				fclose(fp);
			smtp_free_addressees(to);
			smtp_free_addressees(cc);
			smtp_free_addressees(bcc);
		}
		else
		{
			success = FALSE;
		}
	}
	else
	{
		success = FALSE;
	}

	return success;
}

void smtp_set_status(gchar *id, gchar *text, glong level)
{
	ProgressMsg *pmsg;

	if(text)
	{
	}

	if(pmsg = g_malloc0(sizeof(ProgressMsg)))
	{
		pmsg->msg.type = MSG_TYPE_PROGRESS;
		pmsg->who = MSG_WHO_SMTP;
		pmsg->total = smtp_control.Size;
		if(level > -1)
			   pmsg->current = smtp_control.Level = level;
		if(id)
			   strncpy(pmsg->id, id, sizeof(pmsg->id) - 1);
		
		if(text)
			strncpy(pmsg->text, text, sizeof(pmsg->text) - 1);

		pthread_mutex_lock(&euc.q.Mutex);
		list_append(&euc.q.Msgs, pmsg);
		pthread_mutex_unlock(&euc.q.Mutex);
	}
}

/* Callbacks.
 */
void smtp_send_mail_cb(GtkWidget *widget, gpointer data)
{
	/* This function is called as a 
	 * result of the user requesting that
	 * mail be sent, outside of the timer
	 * event.  First, we need to check if there's
	 * a timer running, and if so, kill it.
	 * We also need to account for an already-
	 * invoked SMTP thread, in which case we
	 * ignore this event.
	 */

	/* Check for a running thread.
	 */
	if(smtp_control.th.Thread)
		return;

	/* Check for an existing timer, and kill it.
	 */
	if(smtp_control.Timer)
		gtk_timeout_remove(smtp_control.Timer);
		smtp_control.Timer = 0;

	/* Manually call the timer callback,
	 * which will do the necessary work to
	 * check for outgoing mail and initiate
	 * the thread if necessary.
	 */
	smtp_timer(NULL);
	
	/* The main idle function will take
	 * care of re-instating the timer.
	 */
}


