/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: smtp_control.c
 *   	 SMTP client function server
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <netinet/in.h>
#include <netdb.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "eucalyptus.h"

struct SMTPControl smtp_control;

void smtp_init(void)
{
	memset(&smtp_control, 0, sizeof(smtp_control));
	
	/* Initialize message list.
	 */
	list_init(&smtp_control.Outgoing);
	
	/* Clear out thread attribute info.
	 * Attributes for the thread get set during 
	 * thread creation.
	 */
	pthread_attr_destroy(&smtp_control.th.Attr);
	 
	/* Clear out and initialize mutex info.
	 */
	pthread_mutex_destroy(&smtp_control.th.Mutex);
	pthread_mutex_init(&smtp_control.th.Mutex, NULL);
}

gboolean smtp_thread_start(void)
{
	gboolean success = TRUE;

	/* Check if it's already running.
	 */
	if(smtp_control.th.Thread > 0)
		return TRUE;
			 
	/* Initialize the SMTP thread.
	 */
	pthread_attr_destroy(&smtp_control.th.Attr);
	if(pthread_attr_init(&smtp_control.th.Attr))
	{
		if(!(euc.Flags & EGF_QUIET))
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to initialize SMTP thread!"),
					euc_ok_button, 0, 
					EUCRF_ASYNC);
		success = FALSE;
	}
	else
	{
		/* Set some thread attributes.
		 */
		pthread_attr_setdetachstate(&smtp_control.th.Attr, PTHREAD_CREATE_DETACHED);
		
		/* Start the SMTP thread.
		 */
		if(pthread_create(&smtp_control.th.Thread, &smtp_control.th.Attr, smtp_thread, NULL))
		{	/* Failure :( */
			euc_request(euc.Window,
					_("Eucalyptus error"),
					_("Unable to start SMTP thread!"),
					euc_ok_button, 0,
					EUCRF_ASYNC);
			success = FALSE;
		}
	}

	return success;
}

void smtp_thread_stop(gboolean wait_for_it)
{
	eMsg *msg;

	/* Build the message.
	 */
	if(msg = g_malloc0(sizeof(eMsg)))
	{
		msg->type = MSG_TYPE_DIE;

		/* Send.
		 */
		pthread_mutex_lock(&smtp_control.th.Mutex);
		g_list_prepend(smtp_control.Outgoing.List, msg);
		pthread_mutex_unlock(&smtp_control.th.Mutex);

		/* Wait for response?
		 */
		if(wait_for_it)
		{
			/* FIXME */
		}
	}
}

/* This function gets called when it's time
 * to check for mail to be sent.
 */
gint smtp_timer(gpointer data)
{
	Folder *folder;
	static pthread_attr_t attr;
	struct dirent **namelist;
	glong count, i;
	gchar *fdir;

	/* Get the outgoing folder.
	 */	
	folder = folder_get_special(FTF_OUTGOING);
	if(!folder)
		return TRUE;
		
	/* Check for any messages.
	 */
	if(!folder->Data.MsgCount)
		return TRUE;
		
	/* Get a list of messages, create a message
	 * queue, and start dropping messages into
	 * it.
	 */
	fdir = folder_get_full_path(folder);

	if((count = scandir(fdir, &namelist, smtp_message_file_gatherer, alphasort)) < 1)
	{
		g_free(fdir);
		goto clean_up;
	}
	
	/* Dump the messages into it.
	 */
	for(i=0; i<count; i++)
	{
		QueuedMail *qm;
		SendMsg *smsg;
		
		/* First, check out, if we can access meta-data,
		 * if there is any queuing information for each 
		 * message, so we can determine if the message is
		 * held or queued for a specific send time, etc.
		 */
		/* FIXME */
#if 0
					if(strstr(ap->ap_Info.fib_Comment, "Del;"))
						goto notthisone;
					if(strstr(ap->ap_Info.fib_Comment, "Edt;"))
						goto notthisone;
#endif

		/* Create a queued mail struct for the
		 * window.
		 */
		if(qm = g_malloc0(sizeof(QueuedMail)))
		{
			struct List *list;
			struct stat *st;
			gchar *fname;
			FILE *fp;
			
			fname = path_build(fdir, namelist[i]->d_name, NULL);
			st = path_stat(fname);
			
			strncpy(qm->id, namelist[i]->d_name, sizeof(qm->id) - 1);
			qm->size = st->st_size;
			
			if(fp = fopen(fname, "r"))
			{
				if(list = mailmsg_get_header(GMHT_FILE, fp, NULL))
				{
					HdrNode *hnode;

					if(hnode = header_find_tag("To", list, 0))
					{
						parse_address(hnode->Data, qm->to, sizeof(qm->to), NULL, 0, 0);
						if(!*qm->to)
							parse_address(hnode->Data, NULL, 0, qm->to, sizeof(qm->to), 0);
					}
					if(hnode = header_find_tag("Date", list, 0))
					{
						gchar *c;

						strncpy(qm->date, hnode->Data, sizeof(qm->date) - 1);
						if(c = strchr(qm->date, ':'))
						{
							while(c != qm->date)
							{
								if(*c == ' ')
								{
									*c = '\0';
									break;
								}

								c--;
							}
						}
					}
					if(hnode = header_find_tag("Subject", list, 0))
						strncpy(qm->subject, hnode->Data, sizeof(qm->subject) - 1);

					if(hnode = header_find_tag("Priority", list, 0))
					{
						if(!strcasecmp(hnode->Data, "low"))
						{
							qm->priority = SPT_LOW;
						}
						if(!strcasecmp(hnode->Data, "normal"))
						{
							qm->priority = SPT_NORMAL;
						}
						if(!strcasecmp(hnode->Data, "high"))
						{
							qm->priority = SPT_HIGH;
						}
					}

#if 0
			strncpy(qm->QueueInfo, ap->ap_Info.fib_Comment, sizeof(qm->QueueStatus) - 1);
			if(strstr(qm->QueueInfo, "Sus;"))
			{
				strncpy(qm->QueueStatus, LocaleString(ETXT_SMTP_QUEUESTATUS_SUSPENDED), sizeof(qm->QueueStatus) - 1);
				suspended = TRUE;
			}
			else
			{
				strncpy(qm->QueueStatus, LocaleString(ETXT_SMTP_QUEUESTATUS_WAITING), sizeof(qm->QueueStatus) - 1);
			}

			Enqueue(&control->Outgoing, node);
#endif
					/* Append to list.
					 */
					list_append(&smtp_control.MsgList, qm);
			
					/* Free headers.
					 */
					mailmsg_free_header(list);
				}

				/* Close the file.
				 */
				fclose(fp);
			}

			/* Clean up.
			 */
			g_free(fname);
			g_free(st);
		}

		/* Build the message.
		 */
		if(smsg = g_malloc0(sizeof(SendMsg)))
		{
			smsg->msg.type = MSG_TYPE_SEND;
			strncpy(smsg->filename, namelist[i]->d_name, sizeof(smsg->filename) - 1);
			strncpy(smsg->id, namelist[i]->d_name, sizeof(smsg->id) - 1);

			/* Send.
			 */
			pthread_mutex_lock(&smtp_control.th.Mutex);
			list_append(&smtp_control.Outgoing, smsg);
			pthread_mutex_unlock(&smtp_control.th.Mutex);
		}
	}
	
	g_free(fdir);

	/* Start the SMTP thread.
	 */
	if(!smtp_thread_start())
		return TRUE;

	/* Return "killme" to kill this timer.
	 * It will be re-initiated when the thread
	 * dies and the idle function does its thing.
	 */
	clean_up:
	free(namelist);
	
	return TRUE;
}

/* SMTP message gatherer callback.
 */
int smtp_message_file_gatherer(const struct dirent *de)
{
	/* We only want files that start with
	 * "msg" and are exactly 11 characters
	 * in length.
	 */
	if(!strncmp(de->d_name, "msg", 3) &&
	   (11 == strlen(de->d_name)))
		return TRUE;
		
	return FALSE;
}


